@@ -126,9 +126,9 @@ typedef enum
 
 struct _GdkCursor
 {
-  GdkCursorType type;
+  GdkCursorType GSEAL (type);
   /*< private >*/
-  guint ref_count;
+  guint GSEAL (ref_count);
 };
 
 /* Cursors
@@ -141,12 +141,6 @@ GdkCursor* gdk_cursor_new_for_display	 (GdkDisplay      *display,
 #ifndef GDK_MULTIHEAD_SAFE
 GdkCursor* gdk_cursor_new		 (GdkCursorType	   cursor_type);
 #endif
-GdkCursor* gdk_cursor_new_from_pixmap	 (GdkPixmap	  *source,
-					  GdkPixmap	  *mask,
-					  const GdkColor  *fg,
-					  const GdkColor  *bg,
-					  gint		   x,
-					  gint		   y);
 GdkCursor* gdk_cursor_new_from_pixbuf	 (GdkDisplay      *display,
 					  GdkPixbuf       *pixbuf,
 					  gint             x,
@@ -157,6 +151,7 @@ void        gdk_cursor_unref             (GdkCursor       *cursor);
 GdkCursor*  gdk_cursor_new_from_name	 (GdkDisplay      *display,
 					  const gchar     *name);
 GdkPixbuf*  gdk_cursor_get_image         (GdkCursor       *cursor);
+GdkCursorType gdk_cursor_get_cursor_type (GdkCursor       *cursor);
 
 
 G_END_DECLS
