@@ -22,7 +22,7 @@
 #include "gdkdeviceprivate.h"
 #include "gdkintl.h"
 #include "gdkinternals.h"
-#include "gdkalias.h"
+
 
 typedef struct _GdkAxisInfo GdkAxisInfo;
 
@@ -95,7 +95,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
 				   PROP_DISPLAY,
 				   g_param_spec_object ("display",
                                                         P_("Device Display"),
-                                                        P_("Display to which the device belongs to"),
+                                                        P_("Display which the device belongs to"),
                                                         GDK_TYPE_DISPLAY,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
                                                         G_PARAM_STATIC_STRINGS));
@@ -110,7 +110,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
 				   PROP_DEVICE_MANAGER,
 				   g_param_spec_object ("device-manager",
                                                         P_("Device manager"),
-                                                        P_("Device manager to which the device belongs to"),
+                                                        P_("Device manager which the device belongs to"),
                                                         GDK_TYPE_DEVICE_MANAGER,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
                                                         G_PARAM_STATIC_STRINGS));
@@ -148,7 +148,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
   /**
    * GdkDevice:associated-device:
    *
-   * Associated pointer or keyboard to this device, if any. Devices of type #GDK_DEVICE_TYPE_MASTER
+   * Associated pointer or keyboard with this device, if any. Devices of type #GDK_DEVICE_TYPE_MASTER
    * always come in keyboard/pointer pairs. Other device types will have a %NULL associated device.
    *
    * Since: 3.0
@@ -157,7 +157,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
 				   PROP_ASSOCIATED_DEVICE,
 				   g_param_spec_object ("associated-device",
                                                         P_("Associated device"),
-                                                        P_("Associated pointer or keyboard to this device"),
+                                                        P_("Associated pointer or keyboard with this device"),
                                                         GDK_TYPE_DEVICE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   /**
@@ -202,7 +202,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
   g_object_class_install_property (object_class,
 				   PROP_HAS_CURSOR,
 				   g_param_spec_boolean ("has-cursor",
-                                                         P_("Whether the device has cursor"),
+                                                         P_("Whether the device has a cursor"),
                                                          P_("Whether there is a visible cursor following device motion"),
                                                          FALSE,
                                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
@@ -766,7 +766,7 @@ _gdk_device_set_associated_device (GdkDevice *device,
   GdkDevicePrivate *priv;
 
   g_return_if_fail (GDK_IS_DEVICE (device));
-  g_return_if_fail (GDK_IS_DEVICE (associated));
+  g_return_if_fail (associated == NULL || GDK_IS_DEVICE (associated));
 
   priv = device->priv;
 
@@ -830,7 +830,8 @@ gdk_device_get_n_axes (GdkDevice *device)
  * Returns a #GList of #GdkAtom<!-- -->s, containing the labels for
  * the axes that @device currently has.
  *
- * Returns: A #GList of #GdkAtom<!-- -->s, free with g_list_free().
+ * Returns: (transfer container) (element-type GdkAtom):
+ *     A #GList of #GdkAtom<!-- -->s, free with g_list_free().
  *
  * Since: 3.0
  **/
@@ -1059,7 +1060,7 @@ gdk_device_grab (GdkDevice        *device,
       GdkDisplay *display;
       gulong serial;
 
-      display = gdk_drawable_get_display (window);
+      display = gdk_window_get_display (window);
       serial = _gdk_windowing_window_get_next_serial (display);
 
       _gdk_display_add_device_grab (display,
@@ -1241,7 +1242,7 @@ _gdk_device_translate_window_coord (GdkDevice *device,
     x_min = axis_info_x->min_value;
   else
     {
-      device_width = gdk_screen_get_width (gdk_drawable_get_screen (window));
+      device_width = gdk_screen_get_width (gdk_window_get_screen (window));
       x_min = 0;
     }
 
@@ -1249,12 +1250,13 @@ _gdk_device_translate_window_coord (GdkDevice *device,
     y_min = axis_info_y->min_value;
   else
     {
-      device_height = gdk_screen_get_height (gdk_drawable_get_screen (window));
+      device_height = gdk_screen_get_height (gdk_window_get_screen (window));
       y_min = 0;
     }
 
   window_private = (GdkWindowObject *) window;
-  gdk_drawable_get_size (window, &window_width, &window_height);
+  window_width = gdk_window_get_width (window);
+  window_height = gdk_window_get_height (window);
 
   x_resolution = axis_info_x->resolution;
   y_resolution = axis_info_y->resolution;
@@ -1342,7 +1344,7 @@ _gdk_device_translate_screen_coord (GdkDevice *device,
   if (axis_info.use == GDK_AXIS_X)
     {
       if (axis_width > 0)
-        scale = gdk_screen_get_width (gdk_drawable_get_screen (window)) / axis_width;
+        scale = gdk_screen_get_width (gdk_window_get_screen (window)) / axis_width;
       else
         scale = 1;
 
@@ -1351,7 +1353,7 @@ _gdk_device_translate_screen_coord (GdkDevice *device,
   else
     {
       if (axis_width > 0)
-        scale = gdk_screen_get_height (gdk_drawable_get_screen (window)) / axis_width;
+        scale = gdk_screen_get_height (gdk_window_get_screen (window)) / axis_width;
       else
         scale = 1;
 
@@ -1394,6 +1396,3 @@ _gdk_device_translate_axis (GdkDevice *device,
 
   return TRUE;
 }
-
-#define __GDK_DEVICE_C__
-#include "gdkaliasdef.c"
