@@ -319,7 +319,7 @@ gdk_event_put (const GdkEvent *event)
   g_return_if_fail (event != NULL);
 
   if (event->any.window)
-    display = gdk_drawable_get_display (event->any.window);
+    display = gdk_window_get_display (event->any.window);
   else
     {
       GDK_NOTE (MULTIHEAD,
@@ -540,7 +540,8 @@ gdk_event_free (GdkEvent *event)
     case GDK_DRAG_STATUS:
     case GDK_DROP_START:
     case GDK_DROP_FINISHED:
-      g_object_unref (event->dnd.context);
+      if (event->dnd.context != NULL)
+        g_object_unref (event->dnd.context);
       break;
 
     case GDK_BUTTON_PRESS:
@@ -972,7 +973,7 @@ gdk_event_set_device (GdkEvent  *event,
  * If the event contains a "device" field, this function will return
  * it, else it will return %NULL.
  *
- * Returns: a #GdkDevice, or %NULL.
+ * Returns: (transfer none): a #GdkDevice, or %NULL.
  *
  * Since: 3.0
  **/
@@ -1033,12 +1034,14 @@ gdk_event_get_device (const GdkEvent *event)
       {
         GdkDisplay *display;
         GdkDevice *core_pointer;
+        GdkDeviceManager *device_manager;
 
         g_warning ("Event with type %d not holding a GdkDevice. "
                    "It is most likely synthesized outside Gdk/GTK+\n",
                    event->type);
 
-        display = gdk_drawable_get_display (event->any.window);
+        display = gdk_window_get_display (event->any.window);
+        device_manager = gdk_display_get_device_manager (display);
         core_pointer = gdk_display_get_core_pointer (display);
 
         if (event->type == GDK_KEY_PRESS ||
@@ -1087,7 +1090,7 @@ gdk_event_request_motions (const GdkEventMotion *event)
     {
       gdk_device_get_state (event->device, event->window, NULL, NULL);
       
-      display = gdk_drawable_get_display (event->window);
+      display = gdk_window_get_display (event->window);
       _gdk_display_enable_motion_hints (display, event->device);
     }
 }
@@ -1275,7 +1278,7 @@ gdk_event_get_screen (const GdkEvent *event)
     }
 
   if (event->any.window)
-    return gdk_drawable_get_screen (event->any.window);
+    return gdk_window_get_screen (event->any.window);
 
   return NULL;
 }
@@ -1445,9 +1448,8 @@ gdk_synthesize_window_state (GdkWindow     *window,
   switch (((GdkWindowObject*) window)->window_type)
     {
     case GDK_WINDOW_TOPLEVEL:
-    case GDK_WINDOW_DIALOG:
     case GDK_WINDOW_TEMP: /* ? */
-      gdk_display_put_event (gdk_drawable_get_display (window), &temp_event);
+      gdk_display_put_event (gdk_window_get_display (window), &temp_event);
       break;
       
     case GDK_WINDOW_FOREIGN:
@@ -1512,17 +1514,9 @@ gdk_display_set_double_click_distance (GdkDisplay *display,
   display->double_click_distance = distance;
 }
 
-GType
-gdk_event_get_type (void)
-{
-  static GType our_type = 0;
-  
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static (g_intern_static_string ("GdkEvent"),
-					     (GBoxedCopyFunc)gdk_event_copy,
-					     (GBoxedFreeFunc)gdk_event_free);
-  return our_type;
-}
+G_DEFINE_BOXED_TYPE (GdkEvent, gdk_event,
+                     gdk_event_copy,
+                     gdk_event_free)
 
 /**
  * gdk_setting_get:
