@@ -31,7 +31,6 @@
 
 #include "gdkkeysyms.h"
 #include "gdkinternals.h"
-#include "gdkalias.h"
 
 /* Key handling not part of the keymap */
 
@@ -99,8 +98,5 @@ gdk_keyval_from_name (const gchar *keyval_name)
   if (found != NULL)
     return found->keyval;
   else
-    return GDK_VoidSymbol;
+    return GDK_KEY_VoidSymbol;
 }
-
-#define __GDK_KEYNAMES_C__
-#include "gdkaliasdef.c"
