@@ -23,13 +23,11 @@
 
 #include "config.h"
 #include "gdk.h"		/* For gdk_rectangle_intersect() */
-#include "gdkcolor.h"
 #include "gdkwindow.h"
 #include "gdkscreen.h"
 #include "gdkintl.h"
-#include "gdkalias.h"
 
-static void gdk_screen_dispose      (GObject        *object);
+
 static void gdk_screen_finalize     (GObject        *object);
 static void gdk_screen_set_property (GObject        *object,
 				     guint           prop_id,
@@ -64,7 +62,6 @@ gdk_screen_class_init (GdkScreenClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->dispose = gdk_screen_dispose;
   object_class->finalize = gdk_screen_finalize;
   object_class->set_property = gdk_screen_set_property;
   object_class->get_property = gdk_screen_get_property;
@@ -155,30 +152,6 @@ gdk_screen_init (GdkScreen *screen)
   screen->resolution = -1.;
 }
 
-static void
-gdk_screen_dispose (GObject *object)
-{
-  GdkScreen *screen = GDK_SCREEN (object);
-  gint i;
-
-  for (i = 0; i < 32; ++i)
-    {
-      if (screen->exposure_gcs[i])
-        {
-          g_object_unref (screen->exposure_gcs[i]);
-          screen->exposure_gcs[i] = NULL;
-        }
-
-      if (screen->normal_gcs[i])
-        {
-          g_object_unref (screen->normal_gcs[i]);
-          screen->normal_gcs[i] = NULL;
-        }
-    }
-
-  G_OBJECT_CLASS (gdk_screen_parent_class)->dispose (object);
-}
-
 static void
 gdk_screen_finalize (GObject *object)
 {
@@ -540,6 +513,3 @@ gdk_screen_set_property (GObject      *object,
       break;
     }
 }
-
-#define __GDK_SCREEN_C__
-#include "gdkaliasdef.c"
