@@ -25,7 +25,11 @@
  */
 
 #include "config.h"
+
 #include "gtkeventbox.h"
+
+#include "gtksizerequest.h"
+
 #include "gtkprivate.h"
 #include "gtkintl.h"
 
@@ -53,10 +57,8 @@ static void     gtk_event_box_size_request  (GtkWidget        *widget,
                                              GtkRequisition   *requisition);
 static void     gtk_event_box_size_allocate (GtkWidget        *widget,
                                              GtkAllocation    *allocation);
-static void     gtk_event_box_paint         (GtkWidget        *widget,
-                                             GdkRectangle     *area);
-static gboolean gtk_event_box_expose        (GtkWidget        *widget,
-                                             GdkEventExpose   *event);
+static gboolean gtk_event_box_draw          (GtkWidget        *widget,
+                                             cairo_t          *cr);
 static void     gtk_event_box_set_property  (GObject          *object,
                                              guint             prop_id,
                                              const GValue     *value,
@@ -83,7 +85,7 @@ gtk_event_box_class_init (GtkEventBoxClass *class)
   widget_class->unmap = gtk_event_box_unmap;
   widget_class->size_request = gtk_event_box_size_request;
   widget_class->size_allocate = gtk_event_box_size_allocate;
-  widget_class->expose_event = gtk_event_box_expose;
+  widget_class->draw = gtk_event_box_draw;
 
   g_object_class_install_property (gobject_class,
                                    PROP_VISIBLE_WINDOW,
@@ -366,20 +368,24 @@ gtk_event_box_set_above_child (GtkEventBox *event_box,
 static void
 gtk_event_box_realize (GtkWidget *widget)
 {
+  GtkAllocation allocation;
+  GdkWindow *window;
   GdkWindowAttr attributes;
   gint attributes_mask;
   gint border_width;
   GtkEventBoxPrivate *priv;
   gboolean visible_window;
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   gtk_widget_set_realized (widget, TRUE);
 
   border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
-  
-  attributes.x = widget->allocation.x + border_width;
-  attributes.y = widget->allocation.y + border_width;
-  attributes.width = widget->allocation.width - 2*border_width;
-  attributes.height = widget->allocation.height - 2*border_width;
+
+  attributes.x = allocation.x + border_width;
+  attributes.y = allocation.y + border_width;
+  attributes.width = allocation.width - 2*border_width;
+  attributes.height = allocation.height - 2*border_width;
   attributes.window_type = GDK_WINDOW_CHILD;
   attributes.event_mask = gtk_widget_get_events (widget)
 			| GDK_BUTTON_MOTION_MASK
@@ -395,19 +401,20 @@ gtk_event_box_realize (GtkWidget *widget)
   if (visible_window)
     {
       attributes.visual = gtk_widget_get_visual (widget);
-      attributes.colormap = gtk_widget_get_colormap (widget);
       attributes.wclass = GDK_INPUT_OUTPUT;
       
-      attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
-      
-      widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
-				       &attributes, attributes_mask);
-      gdk_window_set_user_data (widget->window, widget);
+      attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
+
+      window = gdk_window_new (gtk_widget_get_parent_window (widget),
+                               &attributes, attributes_mask);
+      gtk_widget_set_window (widget, window);
+      gdk_window_set_user_data (window, widget);
     }
   else
     {
-      widget->window = gtk_widget_get_parent_window (widget);
-      g_object_ref (widget->window);
+      window = gtk_widget_get_parent_window (widget);
+      gtk_widget_set_window (widget, window);
+      g_object_ref (window);
     }
 
   if (!visible_window || priv->above_child)
@@ -418,16 +425,15 @@ gtk_event_box_realize (GtkWidget *widget)
       else
         attributes_mask = 0;
 
-      priv->event_window = gdk_window_new (widget->window,
+      priv->event_window = gdk_window_new (window,
 					   &attributes, attributes_mask);
       gdk_window_set_user_data (priv->event_window, widget);
     }
 
+  gtk_widget_style_attach (widget);
 
-  widget->style = gtk_style_attach (widget->style, widget->window);
-  
   if (visible_window)
-    gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
+    gtk_style_set_background (gtk_widget_get_style (widget), window, GTK_STATE_NORMAL);
 }
 
 static void
@@ -494,8 +500,8 @@ gtk_event_box_size_request (GtkWidget      *widget,
   if (child && gtk_widget_get_visible (child))
     {
       GtkRequisition child_requisition;
-      
-      gtk_widget_size_request (child, &child_requisition);
+
+      gtk_widget_get_preferred_size (child, &child_requisition, NULL);
 
       requisition->width += child_requisition.width;
       requisition->height += child_requisition.height;
@@ -512,8 +518,10 @@ gtk_event_box_size_allocate (GtkWidget     *widget,
   guint border_width;
   GtkWidget *child;
 
-  widget->allocation = *allocation;
   bin = GTK_BIN (widget);
+
+  gtk_widget_set_allocation (widget, allocation);
+
   border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
   
   if (!gtk_widget_get_has_window (widget))
@@ -541,7 +549,7 @@ gtk_event_box_size_allocate (GtkWidget     *widget,
 				child_allocation.height);
       
       if (gtk_widget_get_has_window (widget))
-	gdk_window_move_resize (widget->window,
+	gdk_window_move_resize (gtk_widget_get_window (widget),
 				allocation->x + border_width,
 				allocation->y + border_width,
 				child_allocation.width,
@@ -553,28 +561,22 @@ gtk_event_box_size_allocate (GtkWidget     *widget,
     gtk_widget_size_allocate (child, &child_allocation);
 }
 
-static void
-gtk_event_box_paint (GtkWidget    *widget,
-		     GdkRectangle *area)
-{
-  if (!gtk_widget_get_app_paintable (widget))
-    gtk_paint_flat_box (widget->style, widget->window,
-			widget->state, GTK_SHADOW_NONE,
-			area, widget, "eventbox",
-			0, 0, -1, -1);
-}
-
 static gboolean
-gtk_event_box_expose (GtkWidget      *widget,
-		     GdkEventExpose *event)
+gtk_event_box_draw (GtkWidget      *widget,
+                    cairo_t        *cr)
 {
-  if (gtk_widget_is_drawable (widget))
-    {
-      if (gtk_widget_get_has_window (widget))
-	gtk_event_box_paint (widget, &event->area);
-
-      GTK_WIDGET_CLASS (gtk_event_box_parent_class)->expose_event (widget, event);
-    }
+  if (gtk_widget_get_has_window (widget) &&
+      !gtk_widget_get_app_paintable (widget))
+    gtk_paint_flat_box (gtk_widget_get_style (widget),
+                        cr,
+			gtk_widget_get_state (widget),
+                        GTK_SHADOW_NONE,
+			widget, "eventbox",
+			0, 0,
+                        gtk_widget_get_allocated_width (widget),
+                        gtk_widget_get_allocated_height (widget));
+  
+  GTK_WIDGET_CLASS (gtk_event_box_parent_class)->draw (widget, cr);
 
   return FALSE;
 }
