@@ -25,9 +25,10 @@
  */
 
 #include "config.h"
+
 #include "gtkimagemenuitem.h"
+
 #include "gtkaccellabel.h"
-#include "gtkintl.h"
 #include "gtkstock.h"
 #include "gtkiconfactory.h"
 #include "gtkimage.h"
@@ -35,6 +36,8 @@
 #include "gtkcontainer.h"
 #include "gtkwindow.h"
 #include "gtkactivatable.h"
+
+#include "gtkintl.h"
 #include "gtkprivate.h"
 
 
@@ -58,7 +61,7 @@ enum {
 
 static GtkActivatableIface *parent_activatable_iface;
 
-static void gtk_image_menu_item_destroy              (GtkObject        *object);
+static void gtk_image_menu_item_destroy              (GtkWidget        *widget);
 static void gtk_image_menu_item_size_request         (GtkWidget        *widget,
                                                       GtkRequisition   *requisition);
 static void gtk_image_menu_item_size_allocate        (GtkWidget        *widget,
@@ -108,13 +111,11 @@ static void
 gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
 {
   GObjectClass *gobject_class = (GObjectClass*) klass;
-  GtkObjectClass *object_class = (GtkObjectClass*) klass;
   GtkWidgetClass *widget_class = (GtkWidgetClass*) klass;
   GtkMenuItemClass *menu_item_class = (GtkMenuItemClass*) klass;
   GtkContainerClass *container_class = (GtkContainerClass*) klass;
 
-  object_class->destroy = gtk_image_menu_item_destroy;
-
+  widget_class->destroy = gtk_image_menu_item_destroy;
   widget_class->screen_changed = gtk_image_menu_item_screen_changed;
   widget_class->size_request = gtk_image_menu_item_size_request;
   widget_class->size_allocate = gtk_image_menu_item_size_allocate;
@@ -194,7 +195,7 @@ gtk_image_menu_item_class_init (GtkImageMenuItemClass *klass)
 						       TRUE,
 						       GTK_PARAM_READWRITE));
 
-  g_type_class_add_private (object_class, sizeof (GtkImageMenuItemPrivate));
+  g_type_class_add_private (klass, sizeof (GtkImageMenuItemPrivate));
 }
 
 static void
@@ -306,16 +307,16 @@ gtk_image_menu_item_map (GtkWidget *widget)
 }
 
 static void
-gtk_image_menu_item_destroy (GtkObject *object)
+gtk_image_menu_item_destroy (GtkWidget *widget)
 {
-  GtkImageMenuItem *image_menu_item = GTK_IMAGE_MENU_ITEM (object);
+  GtkImageMenuItem *image_menu_item = GTK_IMAGE_MENU_ITEM (widget);
   GtkImageMenuItemPrivate *priv = image_menu_item->priv;
 
   if (priv->image)
     gtk_container_remove (GTK_CONTAINER (image_menu_item),
                           priv->image);
 
-  GTK_OBJECT_CLASS (gtk_image_menu_item_parent_class)->destroy (object);
+  GTK_WIDGET_CLASS (gtk_image_menu_item_parent_class)->destroy (widget);
 }
 
 static void
@@ -341,8 +342,8 @@ gtk_image_menu_item_toggle_size_request (GtkMenuItem *menu_item,
     {
       GtkRequisition image_requisition;
       guint toggle_spacing;
-      gtk_widget_get_child_requisition (priv->image,
-                                        &image_requisition);
+
+      gtk_widget_get_preferred_size (priv->image, &image_requisition, NULL);
 
       gtk_widget_style_get (GTK_WIDGET (menu_item),
 			    "toggle-spacing", &toggle_spacing,
@@ -437,8 +438,7 @@ gtk_image_menu_item_size_request (GtkWidget      *widget,
     {
       GtkRequisition child_requisition;
 
-      gtk_widget_size_request (priv->image,
-                               &child_requisition);
+      gtk_widget_get_preferred_size (priv->image, &child_requisition, NULL);
 
       child_width = child_requisition.width;
       child_height = child_requisition.height;
@@ -496,8 +496,7 @@ gtk_image_menu_item_size_allocate (GtkWidget     *widget,
        * come up with a solution that's really better.
        */
 
-      gtk_widget_get_child_requisition (priv->image,
-                                        &child_requisition);
+      gtk_widget_get_preferred_size (priv->image, &child_requisition, NULL);
 
       gtk_widget_get_allocation (widget, &widget_allocation);
 
@@ -966,11 +965,12 @@ gtk_image_menu_item_set_image (GtkImageMenuItem *image_menu_item,
 
 /**
  * gtk_image_menu_item_get_image:
- * @image_menu_item: a #GtkImageMenuItem.
- * @returns: the widget set as image of @image_menu_item.
+ * @image_menu_item: a #GtkImageMenuItem
  *
  * Gets the widget that is currently set as the image of @image_menu_item.
  * See gtk_image_menu_item_set_image().
+ *
+ * Return value: (transfer none): the widget set as image of @image_menu_item
  **/
 GtkWidget*
 gtk_image_menu_item_get_image (GtkImageMenuItem *image_menu_item)
