@@ -25,7 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
@@ -52,50 +52,16 @@ typedef enum
   GTK_NOTEBOOK_TAB_LAST
 } GtkNotebookTab;
 
-typedef struct _GtkNotebook       GtkNotebook;
-typedef struct _GtkNotebookClass  GtkNotebookClass;
-#if !defined (GTK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
-typedef struct _GtkNotebookPage   GtkNotebookPage;
-#endif
+typedef struct _GtkNotebook              GtkNotebook;
+typedef struct _GtkNotebookPrivate       GtkNotebookPrivate;
+typedef struct _GtkNotebookClass         GtkNotebookClass;
 
 struct _GtkNotebook
 {
   GtkContainer container;
-  
-#if !defined (GTK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
-  GtkNotebookPage *GSEAL (cur_page);
-#else
-  gpointer GSEAL (cur_page);
-#endif
-  GList *GSEAL (children);
-  GList *GSEAL (first_tab);		/* The first tab visible (for scrolling notebooks) */
-  GList *GSEAL (focus_tab);
-  
-  GtkWidget *GSEAL (menu);
-  GdkWindow *GSEAL (event_window);
-  
-  guint32 GSEAL (timer);
-  
-  guint16 GSEAL (tab_hborder);
-  guint16 GSEAL (tab_vborder);
-  
-  guint GSEAL (show_tabs)          : 1;
-  guint GSEAL (homogeneous)        : 1;
-  guint GSEAL (show_border)        : 1;
-  guint GSEAL (tab_pos)            : 2;
-  guint GSEAL (scrollable)         : 1;
-  guint GSEAL (in_child)           : 3;
-  guint GSEAL (click_child)        : 3;
-  guint GSEAL (button)             : 2;
-  guint GSEAL (need_timer)         : 1;
-  guint GSEAL (child_has_focus)    : 1;
-  guint GSEAL (have_visible_child) : 1;
-  guint GSEAL (focus_out)          : 1;	/* Flag used by ::move-focus-out implementation */
 
-  guint GSEAL (has_before_previous) : 1;
-  guint GSEAL (has_before_next)     : 1;
-  guint GSEAL (has_after_previous)  : 1;
-  guint GSEAL (has_after_next)      : 1;
+  /* <private> */
+  GtkNotebookPrivate *priv;
 };
 
 struct _GtkNotebookClass
@@ -103,11 +69,7 @@ struct _GtkNotebookClass
   GtkContainerClass parent_class;
 
   void (* switch_page)       (GtkNotebook     *notebook,
-#if !defined (GTK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
-                              GtkNotebookPage *page,
-#else
-                              gpointer         page,
-#endif
+                              GtkWidget       *page,
 			      guint            page_num);
 
   /* Action signals for keybindings */
@@ -138,12 +100,6 @@ struct _GtkNotebookClass
   void (*_gtk_reserved1) (void);
 };
 
-typedef GtkNotebook* (*GtkNotebookWindowCreationFunc) (GtkNotebook *source,
-                                                       GtkWidget   *page,
-                                                       gint         x,
-                                                       gint         y,
-                                                       gpointer     data);
-
 /***********************************************************
  *           Creation, insertion, deletion                 *
  ***********************************************************/
@@ -180,19 +136,9 @@ void gtk_notebook_remove_page       (GtkNotebook *notebook,
  *           Tabs drag and drop                            *
  ***********************************************************/
 
-void gtk_notebook_set_window_creation_hook (GtkNotebookWindowCreationFunc  func,
-					    gpointer                       data,
-                                            GDestroyNotify                 destroy);
-#ifndef GTK_DISABLE_DEPRECATED
-void gtk_notebook_set_group_id             (GtkNotebook *notebook,
-					    gint         group_id);
-gint gtk_notebook_get_group_id             (GtkNotebook *notebook);
-
-#endif /* GTK_DISABLE_DEPRECATED */
-
-void gtk_notebook_set_group                (GtkNotebook *notebook,
-					    gpointer     group);
-gpointer gtk_notebook_get_group            (GtkNotebook *notebook);
+void         gtk_notebook_set_group_name (GtkNotebook *notebook,
+                                          const gchar *group_name);
+const gchar *gtk_notebook_get_group_name (GtkNotebook *notebook);
 
 
 
@@ -224,18 +170,6 @@ gboolean gtk_notebook_get_show_tabs        (GtkNotebook     *notebook);
 void     gtk_notebook_set_tab_pos          (GtkNotebook     *notebook,
 				            GtkPositionType  pos);
 GtkPositionType gtk_notebook_get_tab_pos   (GtkNotebook     *notebook);
-
-#ifndef GTK_DISABLE_DEPRECATED
-void     gtk_notebook_set_homogeneous_tabs (GtkNotebook     *notebook,
-					    gboolean         homogeneous);
-void     gtk_notebook_set_tab_border       (GtkNotebook     *notebook,
-					    guint            border_width);
-void     gtk_notebook_set_tab_hborder      (GtkNotebook     *notebook,
-					    guint            tab_hborder);
-void     gtk_notebook_set_tab_vborder      (GtkNotebook     *notebook,
-					    guint            tab_vborder);
-#endif /* GTK_DISABLE_DEPRECATED */
-
 void     gtk_notebook_set_scrollable       (GtkNotebook     *notebook,
 					    gboolean         scrollable);
 gboolean gtk_notebook_get_scrollable       (GtkNotebook     *notebook);
@@ -273,18 +207,6 @@ void gtk_notebook_set_menu_label_text     (GtkNotebook *notebook,
 					   const gchar *menu_text);
 G_CONST_RETURN gchar *gtk_notebook_get_menu_label_text (GtkNotebook *notebook,
 							GtkWidget   *child);
-#ifndef GTK_DISABLE_DEPRECATED
-void gtk_notebook_query_tab_label_packing (GtkNotebook *notebook,
-					   GtkWidget   *child,
-					   gboolean    *expand,
-					   gboolean    *fill,
-					   GtkPackType *pack_type);
-void gtk_notebook_set_tab_label_packing   (GtkNotebook *notebook,
-					   GtkWidget   *child,
-					   gboolean     expand,
-					   gboolean     fill,
-					   GtkPackType  pack_type);
-#endif
 void gtk_notebook_reorder_child           (GtkNotebook *notebook,
 					   GtkWidget   *child,
 					   gint         position);
@@ -305,11 +227,6 @@ void       gtk_notebook_set_action_widget (GtkNotebook *notebook,
                                            GtkWidget   *widget,
                                            GtkPackType  pack_type);
 
-#ifndef GTK_DISABLE_DEPRECATED
-#define	gtk_notebook_current_page               gtk_notebook_get_current_page
-#define gtk_notebook_set_page                   gtk_notebook_set_current_page
-#endif /* GTK_DISABLE_DEPRECATED */
-
 G_END_DECLS
 
 #endif /* __GTK_NOTEBOOK_H__ */
