@@ -2,7 +2,6 @@
 *tp++ = gdk_axis_use_get_type();
 *tp++ = gdk_byte_order_get_type();
 *tp++ = gdk_color_get_type();
-*tp++ = gdk_colormap_get_type();
 *tp++ = gdk_colorspace_get_type();
 *tp++ = gdk_crossing_mode_get_type();
 *tp++ = gdk_cursor_get_type();
@@ -41,7 +40,6 @@
 *tp++ = gdk_pixbuf_rotation_get_type();
 *tp++ = gdk_pixbuf_simple_anim_get_type();
 *tp++ = gdk_pixbuf_simple_anim_iter_get_type();
-*tp++ = gdk_pixmap_get_type();
 *tp++ = gdk_property_state_get_type();
 *tp++ = gdk_prop_mode_get_type();
 *tp++ = gdk_rectangle_get_type();
@@ -72,6 +70,7 @@
 *tp++ = gtk_action_group_get_type();
 *tp++ = gtk_activatable_get_type();
 *tp++ = gtk_adjustment_get_type();
+*tp++ = gtk_align_get_type();
 *tp++ = gtk_alignment_get_type();
 *tp++ = gtk_application_get_type();
 *tp++ = gtk_arrow_get_type();
@@ -107,7 +106,6 @@
 *tp++ = gtk_cell_renderer_state_get_type();
 *tp++ = gtk_cell_renderer_text_get_type();
 *tp++ = gtk_cell_renderer_toggle_get_type();
-*tp++ = gtk_cell_size_request_get_type();
 *tp++ = gtk_cell_view_get_type();
 *tp++ = gtk_check_button_get_type();
 *tp++ = gtk_check_menu_item_get_type();
@@ -200,8 +198,6 @@
 *tp++ = gtk_notebook_get_type();
 *tp++ = gtk_notebook_tab_get_type();
 *tp++ = gtk_number_up_layout_get_type();
-*tp++ = gtk_object_flags_get_type();
-*tp++ = gtk_object_get_type();
 *tp++ = gtk_offscreen_window_get_type();
 *tp++ = gtk_orientable_get_type();
 *tp++ = gtk_orientation_get_type();
@@ -228,7 +224,6 @@
 *tp++ = gtk_print_quality_get_type();
 *tp++ = gtk_print_settings_get_type();
 *tp++ = gtk_print_status_get_type();
-*tp++ = gtk_private_flags_get_type();
 *tp++ = gtk_progress_bar_get_type();
 *tp++ = gtk_radio_action_get_type();
 *tp++ = gtk_radio_button_get_type();
@@ -271,7 +266,6 @@
 *tp++ = gtk_shadow_type_get_type();
 *tp++ = gtk_size_group_get_type();
 *tp++ = gtk_size_group_mode_get_type();
-*tp++ = gtk_size_request_get_type();
 *tp++ = gtk_size_request_mode_get_type();
 *tp++ = gtk_socket_get_type();
 *tp++ = gtk_sort_type_get_type();
@@ -342,7 +336,6 @@
 *tp++ = gtk_vscale_get_type();
 *tp++ = gtk_vscrollbar_get_type();
 *tp++ = gtk_vseparator_get_type();
-*tp++ = gtk_widget_flags_get_type();
 *tp++ = gtk_widget_get_type();
 *tp++ = gtk_widget_help_type_get_type();
 *tp++ = gtk_window_get_type();
