@@ -33,7 +33,6 @@
 
 #include <gdk/gdk.h>
 #include <gtk/gtkaccelgroup.h>
-#include <gtk/gtkobject.h>
 #include <gtk/gtkadjustment.h>
 #include <gtk/gtkstyle.h>
 #include <gtk/gtksettings.h>
@@ -41,85 +40,6 @@
 
 G_BEGIN_DECLS
 
-/**
- * GtkWidgetFlags:
- * @GTK_TOPLEVEL: widgets without a real parent, as there are #GtkWindow<!-- -->s and
- *  #GtkMenu<!-- -->s have this flag set throughout their lifetime.
- *  Toplevel widgets always contain their own #GdkWindow.
- * @GTK_NO_WINDOW: Indicative for a widget that does not provide its own #GdkWindow.
- *  Visible action (e.g. drawing) is performed on the parent's #GdkWindow.
- * @GTK_REALIZED: Set by gtk_widget_realize(), unset by gtk_widget_unrealize().
- *  A realized widget has an associated #GdkWindow.
- * @GTK_MAPPED: Set by gtk_widget_map(), unset by gtk_widget_unmap().
- *  Only realized widgets can be mapped. It means that gdk_window_show()
- *  has been called on the widgets window(s).
- * @GTK_VISIBLE: Set by gtk_widget_show(), unset by gtk_widget_hide(). Implies that a
- *  widget will be mapped as soon as its parent is mapped.
- * @GTK_SENSITIVE: Set and unset by gtk_widget_set_sensitive().
- *  The sensitivity of a widget determines whether it will receive
- *  certain events (e.g. button or key presses). One premise for
- *  the widget's sensitivity is to have this flag set.
- * @GTK_PARENT_SENSITIVE: Set and unset by gtk_widget_set_sensitive() operations on the
- *  parents of the widget.
- *  This is the second premise for the widget's sensitivity. Once
- *  it has %GTK_SENSITIVE and %GTK_PARENT_SENSITIVE set, its state is
- *  effectively sensitive. This is expressed (and can be examined) by
- *  the #GTK_WIDGET_IS_SENSITIVE macro.
- * @GTK_CAN_FOCUS: Determines whether a widget is able to handle focus grabs.
- * @GTK_HAS_FOCUS: Set by gtk_widget_grab_focus() for widgets that also
- *  have %GTK_CAN_FOCUS set. The flag will be unset once another widget
- *  grabs the focus.
- * @GTK_CAN_DEFAULT: The widget is allowed to receive the default action via
- *  gtk_widget_grab_default() and will reserve space to draw the default if possible
- * @GTK_HAS_DEFAULT: The widget currently is receiving the default action and
- *  should be drawn appropriately if possible
- * @GTK_HAS_GRAB: Set by gtk_grab_add(), unset by gtk_grab_remove(). It means that the
- *  widget is in the grab_widgets stack, and will be the preferred one for
- *  receiving events other than ones of cosmetic value.
- * @GTK_RC_STYLE: Indicates that the widget's style has been looked up through the rc
- *  mechanism. It does not imply that the widget actually had a style
- *  defined through the rc mechanism.
- * @GTK_COMPOSITE_CHILD: Indicates that the widget is a composite child of its parent; see
- *  gtk_widget_push_composite_child(), gtk_widget_pop_composite_child().
- * @GTK_APP_PAINTABLE: Set and unset by gtk_widget_set_app_paintable().
- *  Must be set on widgets whose window the application directly draws on,
- *  in order to keep GTK+ from overwriting the drawn stuff.  See
- *  <xref linkend="app-paintable-widgets"/> for a detailed
- *  description of this flag.
- * @GTK_RECEIVES_DEFAULT: The widget when focused will receive the default action and have
- *  %GTK_HAS_DEFAULT set even if there is a different widget set as default.
- * @GTK_DOUBLE_BUFFERED: Set and unset by gtk_widget_set_double_buffered().
- *  Indicates that exposes done on the widget should be
- *  double-buffered.  See <xref linkend="double-buffering"/> for a
- *  detailed discussion of how double-buffering works in GTK+ and
- *  why you may want to disable it for special cases.
- * @GTK_NO_SHOW_ALL:
- *
- * Tells about certain properties of the widget.
- */
-typedef enum
-{
-  GTK_TOPLEVEL         = 1 << 4,
-  GTK_NO_WINDOW        = 1 << 5,
-  GTK_REALIZED         = 1 << 6,
-  GTK_MAPPED           = 1 << 7,
-  GTK_VISIBLE          = 1 << 8,
-  GTK_SENSITIVE        = 1 << 9,
-  GTK_PARENT_SENSITIVE = 1 << 10,
-  GTK_CAN_FOCUS        = 1 << 11,
-  GTK_HAS_FOCUS        = 1 << 12,
-  GTK_CAN_DEFAULT      = 1 << 13,
-  GTK_HAS_DEFAULT      = 1 << 14,
-  GTK_HAS_GRAB	       = 1 << 15,
-  GTK_RC_STYLE	       = 1 << 16,
-  GTK_COMPOSITE_CHILD  = 1 << 17,
-  GTK_APP_PAINTABLE    = 1 << 18,
-  GTK_RECEIVES_DEFAULT = 1 << 19,
-  GTK_DOUBLE_BUFFERED  = 1 << 20,
-  GTK_NO_SHOW_ALL      = 1 << 21,
-  GTK_MULTIDEVICE      = 1 << 22
-} GtkWidgetFlags;
-
 /* Kinds of widget-specific help */
 typedef enum
 {
@@ -137,68 +57,15 @@ typedef enum
 #define GTK_IS_WIDGET_CLASS(klass)	  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_WIDGET))
 #define GTK_WIDGET_GET_CLASS(obj)         (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_WIDGET, GtkWidgetClass))
 
-/* Macros for extracting the widget flags from GtkWidget.
- */
-/**
- * GTK_WIDGET_FLAGS:
- * @wid: a #GtkWidget.
- *
- * Returns the widget flags from @wid.
- *
- * Deprecated: 2.20: Use the proper function to test individual states:
- * gtk_widget_get_app_paintable(), gtk_widget_get_can_default(),
- * gtk_widget_get_can_focus(), gtk_widget_get_double_buffered(),
- * gtk_widget_has_default(), gtk_widget_is_drawable(),
- * gtk_widget_has_focus(), gtk_widget_has_grab(), gtk_widget_get_mapped(),
- * gtk_widget_get_has_window(), gtk_widget_has_rc_style(),
- * gtk_widget_get_realized(), gtk_widget_get_receives_default(),
- * gtk_widget_get_sensitive(), gtk_widget_is_sensitive(),
- * gtk_widget_is_toplevel() or gtk_widget_get_visible().
- */
-#define GTK_WIDGET_FLAGS(wid)		  (GTK_OBJECT_FLAGS (wid))
-/* FIXME: Deprecating GTK_WIDGET_FLAGS requires fixing GTK internals. */
-
-/* Macros for setting and clearing widget flags.
- */
-/**
- * GTK_WIDGET_SET_FLAGS:
- * @wid: a #GtkWidget.
- * @flag: the flags to set.
- *
- * Turns on certain widget flags.
- *
- * Deprecated: 2.22: Use the proper function instead: gtk_widget_set_app_paintable(),
- *   gtk_widget_set_can_default(), gtk_widget_set_can_focus(),
- *   gtk_widget_set_double_buffered(), gtk_widget_set_has_window(),
- *   gtk_widget_set_mapped(), gtk_widget_set_no_show_all(),
- *   gtk_widget_set_realized(), gtk_widget_set_receives_default(),
- *   gtk_widget_set_sensitive() or gtk_widget_set_visible().
- *
- */
-#define GTK_WIDGET_SET_FLAGS(wid,flag)	  G_STMT_START{ (GTK_WIDGET_FLAGS (wid) |= (flag)); }G_STMT_END
-/* FIXME: Deprecating GTK_WIDGET_SET_FLAGS requires fixing GTK internals. */
-
-/**
- * GTK_WIDGET_UNSET_FLAGS:
- * @wid: a #GtkWidget.
- * @flag: the flags to unset.
- *
- * Turns off certain widget flags.
- *
- * Deprecated: 2.22: Use the proper function instead. See GTK_WIDGET_SET_FLAGS().
- */
-#define GTK_WIDGET_UNSET_FLAGS(wid,flag)  G_STMT_START{ (GTK_WIDGET_FLAGS (wid) &= ~(flag)); }G_STMT_END
-/* FIXME: Deprecating GTK_WIDGET_UNSET_FLAGS requires fixing GTK internals. */
-
 #define GTK_TYPE_REQUISITION              (gtk_requisition_get_type ())
 
 /* forward declaration to avoid excessive includes (and concurrent includes)
  */
 typedef struct _GtkRequisition	   GtkRequisition;
 typedef struct _GtkSelectionData   GtkSelectionData;
+typedef struct _GtkWidgetPrivate   GtkWidgetPrivate;
 typedef struct _GtkWidgetClass	   GtkWidgetClass;
 typedef struct _GtkWidgetAuxInfo   GtkWidgetAuxInfo;
-typedef struct _GtkWidgetShapeInfo GtkWidgetShapeInfo;
 typedef struct _GtkClipboard	   GtkClipboard;
 typedef struct _GtkTooltip         GtkTooltip;
 typedef struct _GtkWindow          GtkWindow;
@@ -249,97 +116,55 @@ struct _GtkRequisition
  */
 struct _GtkWidget
 {
-  /* The object structure needs to be the first
-   *  element in the widget structure in order for
-   *  the object mechanism to work correctly. This
-   *  allows a GtkWidget pointer to be cast to a
-   *  GtkObject pointer.
-   */
-  GtkObject object;
-  
-  /* 16 bits of internally used private flags.
-   * this will be packed into the same 4 byte alignment frame that
-   * state and saved_state go. we therefore don't waste any new
-   * space on this.
-   */
-  guint16 GSEAL (private_flags);
-  
-  /* The state of the widget. There are actually only
-   *  5 widget states (defined in "gtkenums.h").
-   */
-  guint8 GSEAL (state);
-  
-  /* The saved state of the widget. When a widget's state
-   *  is changed to GTK_STATE_INSENSITIVE via
-   *  "gtk_widget_set_state" or "gtk_widget_set_sensitive"
-   *  the old state is kept around in this field. The state
-   *  will be restored once the widget gets sensitive again.
-   */
-  guint8 GSEAL (saved_state);
-  
-  /* The widget's name. If the widget does not have a name
-   *  (the name is NULL), then its name (as returned by
-   *  "gtk_widget_get_name") is its class's name.
-   * Among other things, the widget name is used to determine
-   *  the style to use for a widget.
-   */
-  gchar *GSEAL (name);
-  
-  /*< public >*/
+  GInitiallyUnowned parent_instance;
 
-  /* The style for the widget. The style contains the
-   *  colors the widget should be drawn in for each state
-   *  along with graphics contexts used to draw with and
-   *  the font to use for text.
-   */
-  GtkStyle *GSEAL (style);
-  
-  /* The widget's desired size.
-   */
-  GtkRequisition GSEAL (requisition);
-  
-  /* The widget's allocated size.
-   */
-  GtkAllocation GSEAL (allocation);
-  
-  /* The widget's window or its parent window if it does
-   *  not have a window. (Which will be indicated by the
-   *  GTK_NO_WINDOW flag being set).
-   */
-  GdkWindow *GSEAL (window);
-  
-  /* The widget's parent.
-   */
-  GtkWidget *GSEAL (parent);
+  GtkWidgetPrivate *priv;
 };
 
 /**
  * GtkWidgetClass:
- * @parent_class:
- * @activate_signal:
- * @set_scroll_adjustments_signal:
- *
- * <structfield>activate_signal</structfield>
- * The signal to emit when a widget of this class is activated,
- * gtk_widget_activate() handles the emission. Implementation of this
- * signal is optional.
- *
- *
- * <structfield>set_scroll_adjustment_signal</structfield>
- * This signal is emitted  when a widget of this class is added
- * to a scrolling aware parent, gtk_widget_set_scroll_adjustments()
- * handles the emission.
- * Implementation of this signal is optional.
+ * @parent_class: The object class structure needs to be the first
+ *   element in the widget class structure in order for the class mechanism
+ *   to work correctly. This allows a GtkWidgetClass pointer to be cast to
+ *   a GObjectClass pointer.
+ * @activate_signal: The signal to emit when a widget of this class is
+ *   activated, gtk_widget_activate() handles the emission.
+ *   Implementation of this signal is optional.
+ * @set_scroll_adjustments_signal: This signal is emitted  when a widget of
+ *   this class is added to a scrolling aware parent,
+ *   gtk_widget_set_scroll_adjustments() handles the emission.
+ *   Implementation of this signal is optional.
+ * @adjust_size_request: Convert an initial size request from a widget's
+ *   #GtkSizeRequest virtual method implementations into a size request to
+ *   be used by parent containers in laying out the widget.
+ *   adjust_size_request adjusts <emphasis>from</emphasis> a child widget's
+ *   original request <emphasis>to</emphasis> what a parent container should
+ *   use for layout. The @for_size argument will be -1 if the request should
+ *   not be for a particular size in the opposing orientation, i.e. if the
+ *   request is not height-for-width or width-for-height. If @for_size is
+ *   greater than -1, it is the proposed allocation in the opposing
+ *   orientation that we need the request for. Implementations of
+ *   adjust_size_request should chain up to the default implementation,
+ *   which applies #GtkWidget's margin properties and imposes any values
+ *   from gtk_widget_set_size_request(). Chaining up should be last,
+ *   <emphasis>after</emphasis> your subclass adjusts the request, so
+ *   #GtkWidget can apply constraints and add the margin properly.
+ * @adjust_size_allocation: Convert an initial size allocation assigned
+ *   by a #GtkContainer using gtk_widget_size_allocate(), into an actual
+ *   size allocation to be used by the widget. adjust_size_allocation
+ *   adjusts <emphasis>to</emphasis> a child widget's actual allocation
+ *   <emphasis>from</emphasis> what a parent container computed for the
+ *   child. The adjusted allocation must be entirely within the original
+ *   allocation. In any custom implementation, chain up to the default
+ *   #GtkWidget implementation of this method, which applies the margin
+ *   and alignment properties of #GtkWidget. Chain up
+ *   <emphasis>before</emphasis> performing your own adjustments so your
+ *   own adjustments remove more allocation after the #GtkWidget base
+ *   class has already removed margin and alignment.
  */
 struct _GtkWidgetClass
 {
-  /* The object class structure needs to be the first
-   *  element in the widget class structure in order for
-   *  the class mechanism to work correctly. This allows a
-   *  GtkWidgetClass pointer to be cast to a GtkObjectClass
-   *  pointer.
-   */
-  GtkObjectClass parent_class;
+  GInitiallyUnownedClass parent_class;
 
   /*< public >*/
   
@@ -355,6 +180,7 @@ struct _GtkWidgetClass
 					     GParamSpec **pspecs);
 
   /* basics */
+  void (* destroy)             (GtkWidget        *widget);
   void (* show)		       (GtkWidget        *widget);
   void (* show_all)            (GtkWidget        *widget);
   void (* hide)		       (GtkWidget        *widget);
@@ -381,7 +207,27 @@ struct _GtkWidgetClass
 				gboolean          was_grabbed);
   void (* child_notify)        (GtkWidget	 *widget,
 				GParamSpec       *pspec);
+  gboolean (* draw)	       (GtkWidget	 *widget,
+                                cairo_t          *cr);
   
+  /* size requests */
+  GtkSizeRequestMode (* get_request_mode)               (GtkWidget      *widget);
+
+  void               (* get_preferred_height)           (GtkWidget       *widget,
+                                                         gint            *minimum_height,
+                                                         gint            *natural_height);
+  void               (* get_preferred_width_for_height) (GtkWidget       *widget,
+                                                         gint             height,
+                                                         gint            *minimum_width,
+                                                         gint            *natural_width);
+  void               (* get_preferred_width)            (GtkWidget       *widget,
+                                                         gint            *minimum_width,
+                                                         gint            *natural_width);
+  void               (* get_preferred_height_for_width) (GtkWidget       *widget,
+                                                         gint             width,
+                                                         gint            *minimum_height,
+                                                         gint            *natural_height);
+
   /* Mnemonics */
   gboolean (* mnemonic_activate) (GtkWidget    *widget,
 				  gboolean      group_cycling);
@@ -406,8 +252,6 @@ struct _GtkWidgetClass
 					 GdkEventAny	     *event);
   gboolean (* destroy_event)		(GtkWidget	     *widget,
 					 GdkEventAny	     *event);
-  gboolean (* expose_event)		(GtkWidget	     *widget,
-					 GdkEventExpose      *event);
   gboolean (* key_press_event)		(GtkWidget	     *widget,
 					 GdkEventKey	     *event);
   gboolean (* key_release_event)	(GtkWidget	     *widget,
@@ -522,33 +366,44 @@ struct _GtkWidgetClass
 				       gint        y,
 				       gboolean    keyboard_tooltip,
 				       GtkTooltip *tooltip);
+
+  /*< public >*/
+
+  void         (* adjust_size_request)    (GtkWidget         *widget,
+                                           GtkOrientation     orientation,
+                                           gint               for_size,
+                                           gint              *minimum_size,
+                                           gint              *natural_size);
+  void         (* adjust_size_allocation) (GtkWidget         *widget,
+                                           GtkAllocation     *allocation);
+
+  /*< private >*/
+
   /* Signals without a C default handler class slot:
    * gboolean	(*damage_event)	(GtkWidget      *widget,
    *                             GdkEventExpose *event);
    */
 
   /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
   void (*_gtk_reserved5) (void);
   void (*_gtk_reserved6) (void);
   void (*_gtk_reserved7) (void);
+  void (*_gtk_reserved8) (void);
 };
 
 struct _GtkWidgetAuxInfo
 {
-  gint x;
-  gint y;
   gint width;
   gint height;
 
-  guint x_set : 1;
-  guint y_set : 1;
-};
+  guint   halign : 4;
+  guint   valign : 4;
 
-struct _GtkWidgetShapeInfo
-{
-  gint16     offset_x;
-  gint16     offset_y;
-  GdkBitmap *shape_mask;
+  GtkBorder margin;
 };
 
 GType	   gtk_widget_get_type		  (void) G_GNUC_CONST;
@@ -572,6 +427,8 @@ void	   gtk_widget_unmap		  (GtkWidget	       *widget);
 void	   gtk_widget_realize		  (GtkWidget	       *widget);
 void	   gtk_widget_unrealize		  (GtkWidget	       *widget);
 
+void       gtk_widget_draw                (GtkWidget           *widget,
+                                           cairo_t             *cr);
 /* Queuing draws */
 void	   gtk_widget_queue_draw	  (GtkWidget	       *widget);
 void	   gtk_widget_queue_draw_area	  (GtkWidget	       *widget,
@@ -581,12 +438,36 @@ void	   gtk_widget_queue_draw_area	  (GtkWidget	       *widget,
 					   gint                 height);
 void	   gtk_widget_queue_resize	  (GtkWidget	       *widget);
 void	   gtk_widget_queue_resize_no_redraw (GtkWidget *widget);
+#ifndef GTK_DISABLE_DEPRECATED
 void	   gtk_widget_size_request	  (GtkWidget	       *widget,
 					   GtkRequisition      *requisition);
+#endif
 void	   gtk_widget_size_allocate	  (GtkWidget	       *widget,
 					   GtkAllocation       *allocation);
+
+GtkSizeRequestMode  gtk_widget_get_request_mode               (GtkWidget      *widget);
+void                gtk_widget_get_preferred_width            (GtkWidget      *widget,
+                                                               gint           *minimum_width,
+                                                               gint           *natural_width);
+void                gtk_widget_get_preferred_height_for_width (GtkWidget      *widget,
+                                                               gint            width,
+                                                               gint           *minimum_height,
+                                                               gint           *natural_height);
+void                gtk_widget_get_preferred_height           (GtkWidget      *widget,
+                                                               gint           *minimum_height,
+                                                               gint           *natural_height);
+void                gtk_widget_get_preferred_width_for_height (GtkWidget      *widget,
+                                                               gint            height,
+                                                               gint           *minimum_width,
+                                                               gint           *natural_width);
+void                gtk_widget_get_preferred_size             (GtkWidget      *widget,
+                                                               GtkRequisition *minimum_size,
+                                                               GtkRequisition *natural_size);
+
+#ifndef GTK_DISABLE_DEPRECATED
 void       gtk_widget_get_child_requisition (GtkWidget	       *widget,
 					     GtkRequisition    *requisition);
+#endif
 void	   gtk_widget_add_accelerator	  (GtkWidget           *widget,
 					   const gchar         *accel_signal,
 					   GtkAccelGroup       *accel_group,
@@ -624,8 +505,8 @@ void	   gtk_widget_reparent		  (GtkWidget	       *widget,
 gboolean   gtk_widget_intersect		  (GtkWidget	       *widget,
 					   const GdkRectangle  *area,
 					   GdkRectangle	       *intersection);
-GdkRegion *gtk_widget_region_intersect	  (GtkWidget	       *widget,
-					   const GdkRegion     *region);
+cairo_region_t *gtk_widget_region_intersect	  (GtkWidget	       *widget,
+					   const cairo_region_t     *region);
 
 void	gtk_widget_freeze_child_notify	  (GtkWidget	       *widget);
 void	gtk_widget_child_notify		  (GtkWidget	       *widget,
@@ -712,6 +593,9 @@ void                  gtk_widget_set_window             (GtkWidget    *widget,
                                                          GdkWindow    *window);
 GdkWindow           * gtk_widget_get_window             (GtkWidget    *widget);
 
+int                   gtk_widget_get_allocated_width    (GtkWidget     *widget);
+int                   gtk_widget_get_allocated_height   (GtkWidget     *widget);
+
 void                  gtk_widget_get_allocation         (GtkWidget     *widget,
                                                          GtkAllocation *allocation);
 void                  gtk_widget_set_allocation         (GtkWidget     *widget,
@@ -749,8 +633,9 @@ GdkExtensionMode gtk_widget_get_extension_events (GtkWidget	*widget);
 GtkWidget*   gtk_widget_get_toplevel	(GtkWidget	*widget);
 GtkWidget*   gtk_widget_get_ancestor	(GtkWidget	*widget,
 					 GType		 widget_type);
-GdkColormap* gtk_widget_get_colormap	(GtkWidget	*widget);
 GdkVisual*   gtk_widget_get_visual	(GtkWidget	*widget);
+void         gtk_widget_set_visual	(GtkWidget	*widget,
+                                         GdkVisual      *visual);
 
 GdkScreen *   gtk_widget_get_screen      (GtkWidget *widget);
 gboolean      gtk_widget_has_screen      (GtkWidget *widget);
@@ -759,8 +644,6 @@ GdkWindow *   gtk_widget_get_root_window (GtkWidget *widget);
 GtkSettings*  gtk_widget_get_settings    (GtkWidget *widget);
 GtkClipboard *gtk_widget_get_clipboard   (GtkWidget *widget,
 					  GdkAtom    selection);
-GdkPixmap *   gtk_widget_get_snapshot    (GtkWidget    *widget,
-                                          GdkRectangle *clip_rect);
 
 /* Multidevice support */
 gboolean         gtk_widget_get_support_multidevice (GtkWidget      *widget);
@@ -770,14 +653,27 @@ void             gtk_widget_set_support_multidevice (GtkWidget      *widget,
 /* Accessibility support */
 AtkObject*       gtk_widget_get_accessible               (GtkWidget          *widget);
 
-/* The following functions must not be called on an already
- * realized widget. Because it is possible that somebody
- * can call get_colormap() or get_visual() and save the
- * result, these functions are probably only safe to
- * call in a widget's init() function.
- */
-void         gtk_widget_set_colormap    (GtkWidget      *widget,
-					 GdkColormap    *colormap);
+
+/* Margin and alignment */
+GtkAlign gtk_widget_get_halign        (GtkWidget *widget);
+void     gtk_widget_set_halign        (GtkWidget *widget,
+                                       GtkAlign   align);
+GtkAlign gtk_widget_get_valign        (GtkWidget *widget);
+void     gtk_widget_set_valign        (GtkWidget *widget,
+                                       GtkAlign   align);
+gint     gtk_widget_get_margin_left   (GtkWidget *widget);
+void     gtk_widget_set_margin_left   (GtkWidget *widget,
+                                       gint       margin);
+gint     gtk_widget_get_margin_right  (GtkWidget *widget);
+void     gtk_widget_set_margin_right  (GtkWidget *widget,
+                                       gint       margin);
+gint     gtk_widget_get_margin_top    (GtkWidget *widget);
+void     gtk_widget_set_margin_top    (GtkWidget *widget,
+                                       gint       margin);
+gint     gtk_widget_get_margin_bottom (GtkWidget *widget);
+void     gtk_widget_set_margin_bottom (GtkWidget *widget,
+                                       gint       margin);
+
 
 gint	     gtk_widget_get_events	(GtkWidget	*widget);
 GdkEventMask gtk_widget_get_device_events (GtkWidget	*widget,
@@ -858,10 +754,8 @@ void       gtk_widget_reset_rc_styles   (GtkWidget      *widget);
  * This will override the values that got set by the
  * gtk_widget_set_default_* () functions.
  */
-void	     gtk_widget_push_colormap	     (GdkColormap *cmap);
 void	     gtk_widget_push_composite_child (void);
 void	     gtk_widget_pop_composite_child  (void);
-void	     gtk_widget_pop_colormap	     (void);
 
 /* widget style properties
  */
@@ -887,12 +781,7 @@ void gtk_widget_style_get          (GtkWidget	     *widget,
 
 /* Set certain default values to be used at widget creation time.
  */
-void	     gtk_widget_set_default_colormap (GdkColormap *colormap);
 GtkStyle*    gtk_widget_get_default_style    (void);
-#ifndef GDK_MULTIHEAD_SAFE
-GdkColormap* gtk_widget_get_default_colormap (void);
-GdkVisual*   gtk_widget_get_default_visual   (void);
-#endif
 
 /* Functions for setting directionality for widgets
  */
@@ -907,16 +796,12 @@ GtkTextDirection gtk_widget_get_default_direction (void);
 /* Compositing manager functionality */
 gboolean gtk_widget_is_composited (GtkWidget *widget);
 
-/* Counterpart to gdk_window_shape_combine_mask.
+/* Counterpart to gdk_window_shape_combine_region.
  */
-void	     gtk_widget_shape_combine_mask (GtkWidget *widget,
-					    GdkBitmap *shape_mask,
-					    gint       offset_x,
-					    gint       offset_y);
-void	     gtk_widget_input_shape_combine_mask (GtkWidget *widget,
-						  GdkBitmap *shape_mask,
-						  gint       offset_x,
-						  gint       offset_y);
+void	     gtk_widget_shape_combine_region (GtkWidget *widget,
+                                              cairo_region_t *region);
+void	     gtk_widget_input_shape_combine_region (GtkWidget *widget,
+                                                    cairo_region_t *region);
 
 /* internal function */
 void	     gtk_widget_reset_shapes	   (GtkWidget *widget);
@@ -953,10 +838,22 @@ void       gtk_widget_set_has_tooltip       (GtkWidget   *widget,
 					     gboolean     has_tooltip);
 gboolean   gtk_widget_get_has_tooltip       (GtkWidget   *widget);
 
+gboolean   gtk_cairo_should_draw_window     (cairo_t     *cr,
+                                             GdkWindow   *window);
+void       gtk_cairo_transform_to_window    (cairo_t     *cr,
+                                             GtkWidget   *widget,
+                                             GdkWindow   *window);
+
 GType           gtk_requisition_get_type (void) G_GNUC_CONST;
+GtkRequisition *gtk_requisition_new      (void) G_GNUC_MALLOC;
 GtkRequisition *gtk_requisition_copy     (const GtkRequisition *requisition);
 void            gtk_requisition_free     (GtkRequisition       *requisition);
 
+GdkEventExpose *  _gtk_cairo_get_event                    (cairo_t      *cr);
+
+void              _gtk_widget_draw_internal               (GtkWidget    *widget,
+                                                           cairo_t      *cr,
+                                                           gboolean      clip_to_size);
 void              _gtk_widget_set_has_default             (GtkWidget    *widget,
                                                            gboolean      has_default);
 void              _gtk_widget_set_has_grab                (GtkWidget    *widget,
@@ -987,12 +884,14 @@ void       _gtk_widget_synthesize_crossing (GtkWidget      *from,
                                             GdkDevice      *device,
 					    GdkCrossingMode mode);
 
-GdkColormap* _gtk_widget_peek_colormap (void);
+gpointer     _gtk_widget_peek_request_cache (GtkWidget *widget);
 
 void         _gtk_widget_buildable_finish_accelerator (GtkWidget *widget,
 						       GtkWidget *toplevel,
 						       gpointer   user_data);
 
+gboolean     gtk_widget_in_destruction (GtkWidget *widget);
+
 G_END_DECLS
 
 #endif /* __GTK_WIDGET_H__ */
