@@ -115,14 +115,10 @@ typedef enum
 
 typedef struct
 {
-  GdkDrawable *drawable;
-  GdkGC *gc;
-  
+  cairo_t *cr;
+  HDC dc;
   gint x_offset;
   gint y_offset;
-  
-  /*< private >*/
-  gpointer data;
 } XpDCInfo;
 
 HDC get_window_dc (GtkStyle *style,
@@ -141,6 +137,9 @@ gboolean xp_theme_draw (GdkWindow *win, XpThemeElement element,
                         int height, GtkStateType state_type,
                         GdkRectangle *area);
 gboolean xp_theme_is_drawable (XpThemeElement element);
+gboolean xp_theme_get_element_dimensions (XpThemeElement element,
+                                          GtkStateType state_type,
+                                          gint *cx, gint *cy);
 gboolean xp_theme_get_system_font (XpThemeClass klazz, XpThemeFont fontId, OUT LOGFONTW *lf);
 gboolean xp_theme_get_system_color (XpThemeClass klazz, int colorId, OUT DWORD *pColor);
 gboolean xp_theme_get_system_metric (XpThemeClass klazz, int metricId, OUT int *pVal);
