@@ -1,7 +1,6 @@
 #include <math.h>
 #include <pango/pangocairo.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkprintoperation.h>
 
 static GtkWidget *main_window;
 static char *filename = NULL;
@@ -372,9 +371,6 @@ do_page_setup (GtkAction *action)
 {
   GtkPageSetup *new_page_setup;
 
-  if (settings == NULL)
-    settings = gtk_print_settings_new ();
-  
   new_page_setup = gtk_print_run_page_setup_dialog (GTK_WINDOW (main_window),
 						    page_setup, settings);
 
@@ -437,6 +433,7 @@ typedef struct
 {
   GtkPrintOperation *op;
   GtkPrintOperationPreview *preview;
+  GtkPrintContext   *context;
   GtkWidget         *spin;
   GtkWidget         *area;
   gint               page;
@@ -445,15 +442,26 @@ typedef struct
 } PreviewOp;
 
 static gboolean
-preview_expose (GtkWidget      *widget,
-		GdkEventExpose *event,
-		gpointer        data)
+preview_draw (GtkWidget *widget,
+              cairo_t   *cr,
+              gpointer   data)
 {
   PreviewOp *pop = data;
+  cairo_t *prev_cr;
+  double dpi_x, dpi_y;
 
-  gdk_window_clear (pop->area->window);
+  prev_cr = gtk_print_context_get_cairo_context (pop->context);
+  cairo_reference (prev_cr);
+  dpi_x = gtk_print_context_get_dpi_x (pop->context);
+  dpi_y = gtk_print_context_get_dpi_y (pop->context);
+
+  gtk_print_context_set_cairo_context (pop->context,
+                                       cr, dpi_x, dpi_y);
   gtk_print_operation_preview_render_page (pop->preview,
 					   pop->page - 1);
+  gtk_print_context_set_cairo_context (pop->context,
+                                       prev_cr, dpi_x, dpi_y);
+  cairo_destroy (prev_cr);
 
   return TRUE;
 }
@@ -471,8 +479,8 @@ preview_ready (GtkPrintOperationPreview *preview,
   gtk_spin_button_set_range (GTK_SPIN_BUTTON (pop->spin), 
 			     1.0, n_pages);
 
-  g_signal_connect (pop->area, "expose_event",
-		    G_CALLBACK (preview_expose),
+  g_signal_connect (pop->area, "draw",
+		    G_CALLBACK (preview_draw),
 		    pop);
 
   gtk_widget_queue_draw (pop->area);
@@ -485,6 +493,7 @@ preview_got_page_size (GtkPrintOperationPreview *preview,
 		       gpointer                  data)
 {
   PreviewOp *pop = data;
+  GtkAllocation allocation;
   GtkPaperSize *paper_size;
   double w, h;
   cairo_t *cr;
@@ -495,10 +504,11 @@ preview_got_page_size (GtkPrintOperationPreview *preview,
   w = gtk_paper_size_get_width (paper_size, GTK_UNIT_INCH);
   h = gtk_paper_size_get_height (paper_size, GTK_UNIT_INCH);
 
-  cr = gdk_cairo_create (pop->area->window);
+  cr = gdk_cairo_create (gtk_widget_get_window (pop->area));
 
-  dpi_x = pop->area->allocation.width/w;
-  dpi_y = pop->area->allocation.height/h;
+  gtk_widget_get_allocation (pop->area, &allocation);
+  dpi_x = allocation.width/w;
+  dpi_y = allocation.height/h;
 
   if (fabs (dpi_x - pop->dpi_x) > 0.001 ||
       fabs (dpi_y - pop->dpi_y) > 0.001)
@@ -571,18 +581,16 @@ preview_cb (GtkPrintOperation        *op,
   gtk_widget_set_size_request (GTK_WIDGET (da), width, height);
   gtk_box_pack_start (GTK_BOX (vbox), da, TRUE, TRUE, 0);
 
-  gtk_widget_set_double_buffered (da, FALSE);
-
   gtk_widget_realize (da);
-  
-  cr = gdk_cairo_create (da->window);
 
+  cr = gdk_cairo_create (gtk_widget_get_window (da));
   /* TODO: What dpi to use here? This will be used for pagination.. */
   gtk_print_context_set_cairo_context (context, cr, 72, 72);
   cairo_destroy (cr);
   
-  pop->op = op;
+  pop->op = g_object_ref (op);
   pop->preview = preview;
+  pop->context = context;
   pop->spin = page;
   pop->area = da;
   pop->page = 1;
@@ -637,8 +645,6 @@ print_done (GtkPrintOperation *op,
 
   g_free (print_data->text);
   g_free (print_data->font);
-  g_list_free (print_data->page_breaks);
-  g_object_unref (print_data->layout);
   g_free (print_data);
   
   if (!gtk_print_operation_is_finished (op))
@@ -653,6 +659,15 @@ print_done (GtkPrintOperation *op,
     }
 }
 
+static void
+end_print (GtkPrintOperation *op, GtkPrintContext *context, PrintData *print_data)
+{
+  g_list_free (print_data->page_breaks);
+  print_data->page_breaks = NULL;
+  g_object_unref (print_data->layout);
+  print_data->layout = NULL;
+}
+
 static void
 do_print_or_preview (GtkAction *action, GtkPrintOperationAction print_action)
 {
@@ -675,6 +690,7 @@ do_print_or_preview (GtkAction *action, GtkPrintOperationAction print_action)
     gtk_print_operation_set_default_page_setup (print, page_setup);
   
   g_signal_connect (print, "begin_print", G_CALLBACK (begin_print), print_data);
+  g_signal_connect (print, "end-print", G_CALLBACK (end_print), print_data);
   g_signal_connect (print, "draw_page", G_CALLBACK (draw_page), print_data);
   g_signal_connect (print, "create_custom_widget", G_CALLBACK (create_custom_widget), print_data);
   g_signal_connect (print, "custom_widget_apply", G_CALLBACK (custom_widget_apply), print_data);
@@ -830,9 +846,14 @@ create_window (void)
   GtkWidget *sw;
   GtkActionGroup *actions;
   GError *error;
+  GtkWindowGroup *group;
   
   main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
+  group = gtk_window_group_new ();
+  gtk_window_group_add_window (group, GTK_WINDOW (main_window));
+  g_object_unref (group);
+
   gtk_window_set_default_size (GTK_WINDOW (main_window),
 			       400, 600);
   
@@ -929,14 +950,42 @@ create_window (void)
 int
 main (int argc, char **argv)
 {
+  GError *error = NULL;
+
   g_set_application_name ("Print editor");
   gtk_init (&argc, &argv);
 
+  settings = gtk_print_settings_new_from_file ("print-settings.ini", &error);
+  if (error) {
+    g_print ("Failed to load print settings: %s\n", error->message);
+    g_clear_error (&error);
+
+    settings = gtk_print_settings_new ();
+  }
+  g_assert (settings != NULL);
+
+  page_setup = gtk_page_setup_new_from_file ("page-setup.ini", &error);
+  if (error) {
+    g_print ("Failed to load page setup: %s\n", error->message);
+    g_clear_error (&error);
+  }
+
   create_window ();
 
   if (argc == 2)
     load_file (argv[1]);
   
   gtk_main ();
+
+  if (!gtk_print_settings_to_file (settings, "print-settings.ini", &error)) {
+    g_print ("Failed to save print settings: %s\n", error->message);
+    g_clear_error (&error);
+  }
+  if (page_setup &&
+      !gtk_page_setup_to_file (page_setup, "page-setup.ini", &error)) {
+    g_print ("Failed to save page setup: %s\n", error->message);
+    g_clear_error (&error);
+  }
+
   return 0;
 }
