@@ -19,7 +19,6 @@
 
 #include <string.h>
 
-#undef GTK_DISABLE_DEPRECATED
 #include <gtk/gtk.h>
 
 #include "prop-editor.h"
@@ -50,17 +49,35 @@ signal_removed (gpointer  data,
   g_free (dd);
 }
 
+static gboolean
+is_child_property (GParamSpec *pspec)
+{
+  return g_param_spec_get_qdata (pspec, g_quark_from_string ("is-child-prop")) != NULL;
+}
+
 static void
-g_object_connect_property (GObject *object,
-                           const gchar *prop_name,
-                           GCallback func,
-                           gpointer data,
-                           GObject *alive_object)
+mark_child_property (GParamSpec *pspec)
+{
+  g_param_spec_set_qdata (pspec, g_quark_from_string ("is-child-prop"), 
+			  GINT_TO_POINTER (TRUE));
+}
+
+static void
+g_object_connect_property (GObject     *object,
+			   GParamSpec  *spec,
+                           GCallback    func,
+                           gpointer     data,
+                           GObject     *alive_object)
 {
   GClosure *closure;
-  gchar *with_detail = g_strconcat ("notify::", prop_name, NULL);
+  gchar *with_detail;
   DisconnectData *dd;
 
+  if (is_child_property (spec))
+    with_detail = g_strconcat ("child-notify::", spec->name, NULL);
+  else
+    with_detail = g_strconcat ("notify::", spec->name, NULL);
+
   dd = g_new (DisconnectData, 1);
 
   closure = g_cclosure_new (func, data, NULL);
@@ -84,29 +101,28 @@ g_object_connect_property (GObject *object,
 typedef struct 
 {
   GObject *obj;
-  gchar *prop;
+  GParamSpec *spec;
   gint modified_id;
 } ObjectProperty;
 
 static void
 free_object_property (ObjectProperty *p)
 {
-  g_free (p->prop);
   g_free (p);
 }
 
 static void
-connect_controller (GObject *controller,
+connect_controller (GObject     *controller,
                     const gchar *signal,
-                    GObject *model,
-                    const gchar *prop_name,
-                    GtkSignalFunc func)
+                    GObject     *model,
+		    GParamSpec  *spec,
+                    GCallback    func)
 {
   ObjectProperty *p;
 
   p = g_new (ObjectProperty, 1);
   p->obj = model;
-  p->prop = g_strdup (prop_name);
+  p->spec = spec;
 
   p->modified_id = g_signal_connect_data (controller, signal, func, p,
 					  (GClosureNotify)free_object_property,
@@ -137,7 +153,31 @@ int_modified (GtkAdjustment *adj, gpointer data)
 {
   ObjectProperty *p = data;
 
-  g_object_set (p->obj, p->prop, (int) adj->value, NULL);
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_set (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, (int) adj->value, NULL);
+    }
+  else
+    g_object_set (p->obj, p->spec->name, (int) adj->value, NULL);
+}
+
+static void
+get_property_value (GObject *object, GParamSpec *pspec, GValue *value)
+{
+  if (is_child_property (pspec))
+    {
+      GtkWidget *widget = GTK_WIDGET (object);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_get_property (GTK_CONTAINER (parent),
+					widget, pspec->name, value);
+    }
+  else
+    g_object_get_property (object, pspec->name, value);
 }
 
 static void
@@ -147,7 +187,8 @@ int_changed (GObject *object, GParamSpec *pspec, gpointer data)
   GValue val = { 0, };  
 
   g_value_init (&val, G_TYPE_INT);
-  g_object_get_property (object, pspec->name, &val);
+
+  get_property_value (object, pspec, &val);
 
   if (g_value_get_int (&val) != (int)adj->value)
     {
@@ -164,7 +205,16 @@ uint_modified (GtkAdjustment *adj, gpointer data)
 {
   ObjectProperty *p = data;
 
-  g_object_set (p->obj, p->prop, (guint) adj->value, NULL);
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_set (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, (guint) adj->value, NULL);
+    }
+  else
+    g_object_set (p->obj, p->spec->name, (guint) adj->value, NULL);
 }
 
 static void
@@ -174,7 +224,7 @@ uint_changed (GObject *object, GParamSpec *pspec, gpointer data)
   GValue val = { 0, };  
 
   g_value_init (&val, G_TYPE_UINT);
-  g_object_get_property (object, pspec->name, &val);
+  get_property_value (object, pspec, &val);
 
   if (g_value_get_uint (&val) != (guint)adj->value)
     {
@@ -191,7 +241,16 @@ float_modified (GtkAdjustment *adj, gpointer data)
 {
   ObjectProperty *p = data;
 
-  g_object_set (p->obj, p->prop, (float) adj->value, NULL);
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_set (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, (float) adj->value, NULL);
+    }
+  else
+    g_object_set (p->obj, p->spec->name, (float) adj->value, NULL);
 }
 
 static void
@@ -201,7 +260,7 @@ float_changed (GObject *object, GParamSpec *pspec, gpointer data)
   GValue val = { 0, };  
 
   g_value_init (&val, G_TYPE_FLOAT);
-  g_object_get_property (object, pspec->name, &val);
+  get_property_value (object, pspec, &val);
 
   if (g_value_get_float (&val) != (float) adj->value)
     {
@@ -218,7 +277,16 @@ double_modified (GtkAdjustment *adj, gpointer data)
 {
   ObjectProperty *p = data;
 
-  g_object_set (p->obj, p->prop, (double) adj->value, NULL);
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_set (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, (double) adj->value, NULL);
+    }
+  else
+    g_object_set (p->obj, p->spec->name, (double) adj->value, NULL);
 }
 
 static void
@@ -228,7 +296,7 @@ double_changed (GObject *object, GParamSpec *pspec, gpointer data)
   GValue val = { 0, };  
 
   g_value_init (&val, G_TYPE_DOUBLE);
-  g_object_get_property (object, pspec->name, &val);
+  get_property_value (object, pspec, &val);
 
   if (g_value_get_double (&val) != adj->value)
     {
@@ -248,7 +316,16 @@ string_modified (GtkEntry *entry, gpointer data)
 
   text = gtk_entry_get_text (entry);
 
-  g_object_set (p->obj, p->prop, text, NULL);
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_set (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, text, NULL);
+    }
+  else
+    g_object_set (p->obj, p->spec->name, text, NULL);
 }
 
 static void
@@ -260,7 +337,7 @@ string_changed (GObject *object, GParamSpec *pspec, gpointer data)
   const gchar *text;
   
   g_value_init (&val, G_TYPE_STRING);
-  g_object_get_property (object, pspec->name, &val);
+  get_property_value (object, pspec, &val);
 
   str = g_value_get_string (&val);
   if (str == NULL)
@@ -282,17 +359,27 @@ bool_modified (GtkToggleButton *tb, gpointer data)
 {
   ObjectProperty *p = data;
 
-  g_object_set (p->obj, p->prop, (int) tb->active, NULL);
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_set (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, (int) tb->active, NULL);
+    }
+  else
+    g_object_set (p->obj, p->spec->name, (int) tb->active, NULL);
 }
 
 static void
 bool_changed (GObject *object, GParamSpec *pspec, gpointer data)
 {
   GtkToggleButton *tb = GTK_TOGGLE_BUTTON (data);
+  GtkWidget *child;
   GValue val = { 0, };  
   
   g_value_init (&val, G_TYPE_BOOLEAN);
-  g_object_get_property (object, pspec->name, &val);
+  get_property_value (object, pspec, &val);
 
   if (g_value_get_boolean (&val) != tb->active)
     {
@@ -301,35 +388,42 @@ bool_changed (GObject *object, GParamSpec *pspec, gpointer data)
       unblock_controller (G_OBJECT (tb));
     }
 
-  gtk_label_set_text (GTK_LABEL (GTK_BIN (tb)->child), g_value_get_boolean (&val) ?
-                      "TRUE" : "FALSE");
+  child = gtk_bin_get_child (GTK_BIN (tb));
+  gtk_label_set_text (GTK_LABEL (child),
+                      g_value_get_boolean (&val) ? "TRUE" : "FALSE");
   
   g_value_unset (&val);
 }
 
 
 static void
-enum_modified (GtkOptionMenu *om, gpointer data)
+enum_modified (GtkComboBox *cb, gpointer data)
 {
   ObjectProperty *p = data;
   gint i;
-  GParamSpec *spec;
   GEnumClass *eclass;
   
-  spec = g_object_class_find_property (G_OBJECT_GET_CLASS (p->obj),
-                                       p->prop);
+  eclass = G_ENUM_CLASS (g_type_class_peek (p->spec->value_type));
 
-  eclass = G_ENUM_CLASS (g_type_class_peek (spec->value_type));
-  
-  i = gtk_option_menu_get_history (om);
+  i = gtk_combo_box_get_active (cb);
 
-  g_object_set (p->obj, p->prop, eclass->values[i].value, NULL);
+
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_set (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, eclass->values[i].value, NULL);
+    }
+  else
+    g_object_set (p->obj, p->spec->name, eclass->values[i].value, NULL);
 }
 
 static void
 enum_changed (GObject *object, GParamSpec *pspec, gpointer data)
 {
-  GtkOptionMenu *om = GTK_OPTION_MENU (data);
+  GtkComboBox *cb = GTK_COMBO_BOX (data);
   GValue val = { 0, };  
   GEnumClass *eclass;
   gint i;
@@ -337,7 +431,7 @@ enum_changed (GObject *object, GParamSpec *pspec, gpointer data)
   eclass = G_ENUM_CLASS (g_type_class_peek (pspec->value_type));
   
   g_value_init (&val, pspec->value_type);
-  g_object_get_property (object, pspec->name, &val);
+  get_property_value (object, pspec, &val);
 
   i = 0;
   while (i < eclass->n_values)
@@ -347,17 +441,88 @@ enum_changed (GObject *object, GParamSpec *pspec, gpointer data)
       ++i;
     }
   
-  if (gtk_option_menu_get_history (om) != i)
+  if (gtk_combo_box_get_active (cb) != i)
     {
-      block_controller (G_OBJECT (om));
-      gtk_option_menu_set_history (om, i);
-      unblock_controller (G_OBJECT (om));
+      block_controller (G_OBJECT (cb));
+      gtk_combo_box_set_active (cb, i);
+      unblock_controller (G_OBJECT (cb));
     }
   
   g_value_unset (&val);
 
 }
 
+static void
+flags_modified (GtkCheckButton *button, gpointer data)
+{
+  ObjectProperty *p = data;
+  gboolean active;
+  GFlagsClass *fclass;
+  guint flags;
+  gint i;
+  
+  fclass = G_FLAGS_CLASS (g_type_class_peek (p->spec->value_type));
+  
+  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+  i = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (button), "index"));
+
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_get (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, &flags, NULL);
+      if (active)
+        flags |= fclass->values[i].value;
+      else
+        flags &= ~fclass->values[i].value;
+
+      gtk_container_child_set (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, flags, NULL);
+    }
+  else
+    {
+      g_object_get (p->obj, p->spec->name, &flags, NULL);
+
+      if (active)
+        flags |= fclass->values[i].value;
+      else
+        flags &= ~fclass->values[i].value;
+
+      g_object_set (p->obj, p->spec->name, flags, NULL);
+    }
+}
+
+static void
+flags_changed (GObject *object, GParamSpec *pspec, gpointer data)
+{
+  GList *children, *c;
+  GValue val = { 0, };  
+  GFlagsClass *fclass;
+  guint flags;
+  gint i;
+
+  fclass = G_FLAGS_CLASS (g_type_class_peek (pspec->value_type));
+  
+  g_value_init (&val, pspec->value_type);
+  get_property_value (object, pspec, &val);
+  flags = g_value_get_flags (&val);
+  g_value_unset (&val);
+
+  children = gtk_container_get_children (GTK_CONTAINER (data));
+
+  for (c = children, i = 0; c; c = c->next, i++)
+    {
+      block_controller (G_OBJECT (c->data));
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (c->data),
+                                    (fclass->values[i].value & flags) != 0);
+      unblock_controller (G_OBJECT (c->data));
+    }
+
+  g_list_free (children);
+}
+
 static gunichar
 unichar_get_value (GtkEntry *entry)
 {
@@ -375,7 +540,16 @@ unichar_modified (GtkEntry *entry, gpointer data)
   ObjectProperty *p = data;
   gunichar val = unichar_get_value (entry);
 
-  g_object_set (p->obj, p->prop, val, NULL);
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_set (GTK_CONTAINER (parent), 
+			       widget, p->spec->name, val, NULL);
+    }
+  else
+    g_object_set (p->obj, p->spec->name, val, NULL);
 }
 
 static void
@@ -384,10 +558,13 @@ unichar_changed (GObject *object, GParamSpec *pspec, gpointer data)
   GtkEntry *entry = GTK_ENTRY (data);
   gunichar new_val;
   gunichar old_val = unichar_get_value (entry);
+  GValue val = { 0, };
   gchar buf[7];
   gint len;
-
-  g_object_get (object, pspec->name, &new_val, NULL);
+  
+  g_value_init (&val, pspec->value_type);
+  get_property_value (object, pspec, &val);
+  new_val = (gunichar)g_value_get_uint (&val);
 
   if (new_val != old_val)
     {
@@ -418,13 +595,26 @@ pointer_changed (GObject *object, GParamSpec *pspec, gpointer data)
   g_free (str);
 }
 
+static gchar *
+object_label (GObject *obj, GParamSpec *pspec)
+{
+  const gchar *name;
+
+  if (obj)
+    name = g_type_name (G_TYPE_FROM_INSTANCE (obj));
+  else if (pspec)
+    name = g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec));
+  else
+    name = "unknown";
+  return g_strdup_printf ("Object: %p (%s)", obj, name);
+}
+
 static void
 object_changed (GObject *object, GParamSpec *pspec, gpointer data)
 {
   GtkWidget *label, *button;
   gchar *str;
   GObject *obj;
-  const gchar *name;
   
   GList *children = gtk_container_get_children (GTK_CONTAINER (data)); 
   label = GTK_WIDGET (children->data);
@@ -432,15 +622,14 @@ object_changed (GObject *object, GParamSpec *pspec, gpointer data)
   g_object_get (object, pspec->name, &obj, NULL);
   g_list_free (children);
 
-  if (obj)
-    name = g_type_name (G_TYPE_FROM_INSTANCE (obj));
-  else
-    name = "unknown";
-  str = g_strdup_printf ("Object: %p (%s)", obj, name);
+  str = object_label (obj, pspec);
   
   gtk_label_set_text (GTK_LABEL (label), str);
   gtk_widget_set_sensitive (button, G_IS_OBJECT (obj));
 
+  if (obj)
+    g_object_unref (obj);
+
   g_free (str);
 }
 
@@ -470,8 +659,54 @@ object_properties (GtkWidget *button,
     create_prop_editor (obj, 0);
 }
  
+static void
+color_modified (GtkColorButton *cb, gpointer data)
+{
+  ObjectProperty *p = data;
+  GdkColor color;
+
+  gtk_color_button_get_color (cb, &color);
+
+  if (is_child_property (p->spec))
+    {
+      GtkWidget *widget = GTK_WIDGET (p->obj);
+      GtkWidget *parent = gtk_widget_get_parent (widget);
+
+      gtk_container_child_set (GTK_CONTAINER (parent),
+			       widget, p->spec->name, &color, NULL);
+    }
+  else
+    g_object_set (p->obj, p->spec->name, &color, NULL);
+}
+
+static void
+color_changed (GObject *object, GParamSpec *pspec, gpointer data)
+{
+  GtkColorButton *cb = GTK_COLOR_BUTTON (data);
+  GValue val = { 0, };
+  GdkColor *color;
+  GdkColor cb_color;
+
+  g_value_init (&val, GDK_TYPE_COLOR);
+  get_property_value (object, pspec, &val);
+
+  color = g_value_get_boxed (&val);
+  gtk_color_button_get_color (cb, &cb_color);
+
+  if (color != NULL && !gdk_color_equal (color, &cb_color))
+    {
+      block_controller (G_OBJECT (cb));
+      gtk_color_button_set_color (cb, color);
+      unblock_controller (G_OBJECT (cb));
+    }
+
+  g_value_unset (&val);
+}
+
 static GtkWidget *
-property_widget (GObject *object, GParamSpec *spec, gboolean can_modify)
+property_widget (GObject    *object, 
+		 GParamSpec *spec, 
+		 gboolean    can_modify)
 {
   GtkWidget *prop_edit;
   GtkAdjustment *adj;
@@ -480,148 +715,161 @@ property_widget (GObject *object, GParamSpec *spec, gboolean can_modify)
 
   if (type == G_TYPE_PARAM_INT)
     {
-      adj = GTK_ADJUSTMENT (gtk_adjustment_new (G_PARAM_SPEC_INT (spec)->default_value,
-						G_PARAM_SPEC_INT (spec)->minimum,
-						G_PARAM_SPEC_INT (spec)->maximum,
-						1,
-						MAX ((G_PARAM_SPEC_INT (spec)->maximum -
-						      G_PARAM_SPEC_INT (spec)->minimum) / 10, 1),
-						0.0));
-      
+      adj = gtk_adjustment_new (G_PARAM_SPEC_INT (spec)->default_value,
+                                G_PARAM_SPEC_INT (spec)->minimum,
+                                G_PARAM_SPEC_INT (spec)->maximum,
+                                1,
+                                MAX ((G_PARAM_SPEC_INT (spec)->maximum - G_PARAM_SPEC_INT (spec)->minimum) / 10, 1),
+                                0.0);
+
       prop_edit = gtk_spin_button_new (adj, 1.0, 0);
       
-      g_object_connect_property (object, spec->name,
-				 G_CALLBACK (int_changed),
+      g_object_connect_property (object, spec, 
+				 G_CALLBACK (int_changed), 
 				 adj, G_OBJECT (adj));
       
       if (can_modify)
 	connect_controller (G_OBJECT (adj), "value_changed",
-			    object, spec->name, (GtkSignalFunc) int_modified);
+			    object, spec, G_CALLBACK (int_modified));
     }
   else if (type == G_TYPE_PARAM_UINT)
     {
-      adj = GTK_ADJUSTMENT (
-			    gtk_adjustment_new (G_PARAM_SPEC_UINT (spec)->default_value,
-						G_PARAM_SPEC_UINT (spec)->minimum,
-						G_PARAM_SPEC_UINT (spec)->maximum,
-						1,
-						MAX ((G_PARAM_SPEC_UINT (spec)->maximum -
-						      G_PARAM_SPEC_UINT (spec)->minimum) / 10, 1),
-						0.0));
-      
+      adj = gtk_adjustment_new (G_PARAM_SPEC_UINT (spec)->default_value,
+                                G_PARAM_SPEC_UINT (spec)->minimum,
+                                G_PARAM_SPEC_UINT (spec)->maximum,
+                                1,
+                                MAX ((G_PARAM_SPEC_UINT (spec)->maximum - G_PARAM_SPEC_UINT (spec)->minimum) / 10, 1),
+                                0.0);
+
       prop_edit = gtk_spin_button_new (adj, 1.0, 0);
       
-      g_object_connect_property (object, spec->name,
-				 G_CALLBACK (uint_changed),
+      g_object_connect_property (object, spec, 
+				 G_CALLBACK (uint_changed), 
 				 adj, G_OBJECT (adj));
       
       if (can_modify)
 	connect_controller (G_OBJECT (adj), "value_changed",
-			    object, spec->name, (GtkSignalFunc) uint_modified);
+			    object, spec, G_CALLBACK (uint_modified));
     }
   else if (type == G_TYPE_PARAM_FLOAT)
     {
+      adj = gtk_adjustment_new (G_PARAM_SPEC_FLOAT (spec)->default_value,
+                                G_PARAM_SPEC_FLOAT (spec)->minimum,
+                                G_PARAM_SPEC_FLOAT (spec)->maximum,
+                                0.1,
+                                MAX ((G_PARAM_SPEC_FLOAT (spec)->maximum - G_PARAM_SPEC_FLOAT (spec)->minimum) / 10, 0.1),
+                                0.0);
 
-      adj = GTK_ADJUSTMENT (gtk_adjustment_new (G_PARAM_SPEC_FLOAT (spec)->default_value,
-						G_PARAM_SPEC_FLOAT (spec)->minimum,
-						G_PARAM_SPEC_FLOAT (spec)->maximum,
-						0.1,
-						MAX ((G_PARAM_SPEC_FLOAT (spec)->maximum -
-						      G_PARAM_SPEC_FLOAT (spec)->minimum) / 10, 0.1),
-						0.0));
-      
       prop_edit = gtk_spin_button_new (adj, 0.1, 2);
       
-      g_object_connect_property (object, spec->name,
-				 G_CALLBACK (float_changed),
+      g_object_connect_property (object, spec, 
+				 G_CALLBACK (float_changed), 
 				 adj, G_OBJECT (adj));
       
       if (can_modify)
 	connect_controller (G_OBJECT (adj), "value_changed",
-			    object, spec->name, (GtkSignalFunc) float_modified);
+			    object, spec, G_CALLBACK (float_modified));
     }
   else if (type == G_TYPE_PARAM_DOUBLE)
     {
-      adj = GTK_ADJUSTMENT (gtk_adjustment_new (G_PARAM_SPEC_DOUBLE (spec)->default_value,
-						G_PARAM_SPEC_DOUBLE (spec)->minimum,
-						G_PARAM_SPEC_DOUBLE (spec)->maximum,
-						0.1,
-						MAX ((G_PARAM_SPEC_DOUBLE (spec)->maximum -
-						      G_PARAM_SPEC_DOUBLE (spec)->minimum) / 10, 0.1),
-						0.0));
-      
+      adj = gtk_adjustment_new (G_PARAM_SPEC_DOUBLE (spec)->default_value,
+                                G_PARAM_SPEC_DOUBLE (spec)->minimum,
+                                G_PARAM_SPEC_DOUBLE (spec)->maximum,
+                                0.1,
+                                MAX ((G_PARAM_SPEC_DOUBLE (spec)->maximum - G_PARAM_SPEC_DOUBLE (spec)->minimum) / 10, 0.1),
+                                0.0);
+
       prop_edit = gtk_spin_button_new (adj, 0.1, 2);
       
-      g_object_connect_property (object, spec->name,
-				 G_CALLBACK (double_changed),
+      g_object_connect_property (object, spec, 
+				 G_CALLBACK (double_changed), 
 				 adj, G_OBJECT (adj));
       
       if (can_modify)
 	connect_controller (G_OBJECT (adj), "value_changed",
-			    object, spec->name, (GtkSignalFunc) double_modified);
+			    object, spec, G_CALLBACK (double_modified));
     }
   else if (type == G_TYPE_PARAM_STRING)
     {
       prop_edit = gtk_entry_new ();
       
-      g_object_connect_property (object, spec->name,
+      g_object_connect_property (object, spec, 
 				 G_CALLBACK (string_changed),
 				 prop_edit, G_OBJECT (prop_edit));
       
       if (can_modify)
 	connect_controller (G_OBJECT (prop_edit), "changed",
-			    object, spec->name, (GtkSignalFunc) string_modified);
+			    object, spec, G_CALLBACK (string_modified));
     }
   else if (type == G_TYPE_PARAM_BOOLEAN)
     {
       prop_edit = gtk_toggle_button_new_with_label ("");
       
-      g_object_connect_property (object, spec->name,
+      g_object_connect_property (object, spec, 
 				 G_CALLBACK (bool_changed),
 				 prop_edit, G_OBJECT (prop_edit));
       
       if (can_modify)
 	connect_controller (G_OBJECT (prop_edit), "toggled",
-			    object, spec->name, (GtkSignalFunc) bool_modified);
+			    object, spec, G_CALLBACK (bool_modified));
     }
   else if (type == G_TYPE_PARAM_ENUM)
     {
       {
-	GtkWidget *menu;
 	GEnumClass *eclass;
 	gint j;
 	
-	prop_edit = gtk_option_menu_new ();
-	
-	menu = gtk_menu_new ();
+	prop_edit = gtk_combo_box_new_text ();
 	
 	eclass = G_ENUM_CLASS (g_type_class_ref (spec->value_type));
 	
 	j = 0;
 	while (j < eclass->n_values)
 	  {
-	    GtkWidget *mi;
-	    
-	    mi = gtk_menu_item_new_with_label (eclass->values[j].value_name);
-	    
-	    gtk_widget_show (mi);
-	    
-	    gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-	    
+	    gtk_combo_box_append_text (GTK_COMBO_BOX (prop_edit),
+	                               eclass->values[j].value_name);
 	    ++j;
 	  }
 	
 	g_type_class_unref (eclass);
 	
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (prop_edit), menu);
-	
-	g_object_connect_property (object, spec->name,
+	g_object_connect_property (object, spec,
 				   G_CALLBACK (enum_changed),
 				   prop_edit, G_OBJECT (prop_edit));
 	
 	if (can_modify)
 	  connect_controller (G_OBJECT (prop_edit), "changed",
-			      object, spec->name, (GtkSignalFunc) enum_modified);
+			      object, spec, G_CALLBACK (enum_modified));
+      }
+    }
+  else if (type == G_TYPE_PARAM_FLAGS)
+    {
+      {
+	GFlagsClass *fclass;
+	gint j;
+	
+	prop_edit = gtk_vbox_new (FALSE, 0);
+	
+	fclass = G_FLAGS_CLASS (g_type_class_ref (spec->value_type));
+	
+	for (j = 0; j < fclass->n_values; j++)
+	  {
+	    GtkWidget *b;
+	    
+	    b = gtk_check_button_new_with_label (fclass->values[j].value_name);
+            g_object_set_data (G_OBJECT (b), "index", GINT_TO_POINTER (j));
+	    gtk_widget_show (b);
+	    gtk_box_pack_start (GTK_BOX (prop_edit), b, FALSE, FALSE, 0);
+	    if (can_modify) 
+	      connect_controller (G_OBJECT (b), "toggled",
+	     	                  object, spec, G_CALLBACK (flags_modified));
+	  }
+	
+	g_type_class_unref (fclass);
+	
+	g_object_connect_property (object, spec,
+				   G_CALLBACK (flags_changed),
+				   prop_edit, G_OBJECT (prop_edit));
       }
     }
   else if (type == G_TYPE_PARAM_UNICHAR)
@@ -629,19 +877,19 @@ property_widget (GObject *object, GParamSpec *spec, gboolean can_modify)
       prop_edit = gtk_entry_new ();
       gtk_entry_set_max_length (GTK_ENTRY (prop_edit), 1);
       
-      g_object_connect_property (object, spec->name,
+      g_object_connect_property (object, spec,
 				 G_CALLBACK (unichar_changed),
 				 prop_edit, G_OBJECT (prop_edit));
       
       if (can_modify)
 	connect_controller (G_OBJECT (prop_edit), "changed",
-			    object, spec->name, (GtkSignalFunc) unichar_modified);
+			    object, spec, G_CALLBACK (unichar_modified));
     }
   else if (type == G_TYPE_PARAM_POINTER)
     {
       prop_edit = gtk_label_new ("");
       
-      g_object_connect_property (object, spec->name,
+      g_object_connect_property (object, spec, 
 				 G_CALLBACK (pointer_changed),
 				 prop_edit, G_OBJECT (prop_edit));
     }
@@ -661,10 +909,23 @@ property_widget (GObject *object, GParamSpec *spec, gboolean can_modify)
       gtk_container_add (GTK_CONTAINER (prop_edit), label);
       gtk_container_add (GTK_CONTAINER (prop_edit), button);
       
-      g_object_connect_property (object, spec->name,
+      g_object_connect_property (object, spec,
 				 G_CALLBACK (object_changed),
 				 prop_edit, G_OBJECT (label));
     }
+  else if (type == G_TYPE_PARAM_BOXED &&
+           G_PARAM_SPEC_VALUE_TYPE (spec) == GDK_TYPE_COLOR)
+    {
+      prop_edit = gtk_color_button_new ();
+
+      g_object_connect_property (object, spec,
+				 G_CALLBACK (color_changed),
+				 prop_edit, G_OBJECT (prop_edit));
+
+      if (can_modify)
+	connect_controller (G_OBJECT (prop_edit), "color-set",
+			    object, spec, G_CALLBACK (color_modified));
+    }
   else
     {  
       msg = g_strdup_printf ("uneditable property type: %s",
@@ -678,9 +939,8 @@ property_widget (GObject *object, GParamSpec *spec, gboolean can_modify)
 }
 
 static GtkWidget *
-properties_from_type (GObject     *object,
-		      GType        type,
-		      GtkTooltips *tips)
+properties_from_type (GObject *object,
+		      GType    type)
 {
   GtkWidget *prop_edit;
   GtkWidget *label;
@@ -688,7 +948,7 @@ properties_from_type (GObject     *object,
   GtkWidget *vbox;
   GtkWidget *table;
   GParamSpec **specs;
-  gint n_specs;
+  guint n_specs;
   int i;
 
   if (G_TYPE_IS_INTERFACE (type))
@@ -702,8 +962,10 @@ properties_from_type (GObject     *object,
       specs = g_object_class_list_properties (class, &n_specs);
     }
         
-  if (n_specs == 0)
+  if (n_specs == 0) {
+    g_free (specs);
     return NULL;
+  }
   
   table = gtk_table_new (n_specs, 2, FALSE);
   gtk_table_set_col_spacing (GTK_TABLE (table), 0, 10);
@@ -747,8 +1009,8 @@ properties_from_type (GObject     *object,
             gtk_widget_set_sensitive (prop_edit, FALSE);
 
 	  if (g_param_spec_get_blurb (spec))
-	    gtk_tooltips_set_tip (tips, prop_edit, g_param_spec_get_blurb (spec), NULL);
-	  
+	    gtk_widget_set_tooltip_text (prop_edit, g_param_spec_get_blurb (spec));
+
           /* set initial value */
           g_object_notify (object, spec->name);
         }
@@ -771,6 +1033,204 @@ properties_from_type (GObject     *object,
   return sw;
 }
 
+static GtkWidget *
+child_properties_from_object (GObject *object)
+{
+  GtkWidget *prop_edit;
+  GtkWidget *label;
+  GtkWidget *sw;
+  GtkWidget *vbox;
+  GtkWidget *table;
+  GtkWidget *parent;
+  GParamSpec **specs;
+  guint n_specs;
+  gint i;
+
+  if (!GTK_IS_WIDGET (object))
+    return NULL;
+
+  parent = gtk_widget_get_parent (GTK_WIDGET (object));
+
+  if (!parent)
+    return NULL;
+
+  specs = gtk_container_class_list_child_properties (G_OBJECT_GET_CLASS (parent), &n_specs);
+
+  table = gtk_table_new (n_specs, 2, FALSE);
+  gtk_table_set_col_spacing (GTK_TABLE (table), 0, 10);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 3);
+
+  i = 0;
+  while (i < n_specs)
+    {
+      GParamSpec *spec = specs[i];
+      gboolean can_modify;
+      
+      prop_edit = NULL;
+
+      can_modify = ((spec->flags & G_PARAM_WRITABLE) != 0 &&
+                    (spec->flags & G_PARAM_CONSTRUCT_ONLY) == 0);
+      
+      if ((spec->flags & G_PARAM_READABLE) == 0)
+        {
+          /* can't display unreadable properties */
+          ++i;
+          continue;
+        }
+      
+      label = gtk_label_new (g_param_spec_get_nick (spec));
+      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+      gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, i, i + 1);
+      
+      mark_child_property (spec);
+      prop_edit = property_widget (object, spec, can_modify);
+      gtk_table_attach_defaults (GTK_TABLE (table), prop_edit, 1, 2, i, i + 1);
+
+      if (prop_edit)
+        {
+          if (!can_modify)
+            gtk_widget_set_sensitive (prop_edit, FALSE);
+
+	  if (g_param_spec_get_blurb (spec))
+	    gtk_widget_set_tooltip_text (prop_edit, g_param_spec_get_blurb (spec));
+
+          /* set initial value */
+          gtk_widget_child_notify (GTK_WIDGET (object), spec->name);
+        }
+      
+      ++i;
+    }
+
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+
+  sw = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), vbox);
+
+  g_free (specs);
+
+  return sw;
+}
+
+static void
+child_properties (GtkWidget *button, 
+		  GObject   *object)
+{
+  create_prop_editor (object, 0);
+}
+
+static GtkWidget *
+children_from_object (GObject *object)
+{
+  GList *children, *c;
+  GtkWidget *table, *label, *prop_edit, *button, *vbox, *sw;
+  gchar *str;
+  gint i;
+
+  if (!GTK_IS_CONTAINER (object))
+    return NULL;
+
+  children = gtk_container_get_children (GTK_CONTAINER (object));
+
+  table = gtk_table_new (g_list_length (children), 2, FALSE);
+  gtk_table_set_col_spacing (GTK_TABLE (table), 0, 10);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 3);
+ 
+  for (c = children, i = 0; c; c = c->next, i++)
+    {
+      object = c->data;
+
+      label = gtk_label_new ("Child");
+      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+      gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, i, i + 1);
+
+      prop_edit = gtk_hbox_new (FALSE, 5);
+
+      str = object_label (object, NULL);
+      label = gtk_label_new (str);
+      g_free (str);
+      button = gtk_button_new_with_label ("Properties");
+      g_signal_connect (button, "clicked",
+                        G_CALLBACK (child_properties),
+                        object);
+
+      gtk_container_add (GTK_CONTAINER (prop_edit), label);
+      gtk_container_add (GTK_CONTAINER (prop_edit), button);
+
+      gtk_table_attach_defaults (GTK_TABLE (table), prop_edit, 1, 2, i, i + 1);
+    }
+
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+
+  sw = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), vbox);
+
+  g_list_free (children);
+
+  return sw;
+}
+
+static GtkWidget *
+cells_from_object (GObject *object)
+{
+  GList *cells, *c;
+  GtkWidget *table, *label, *prop_edit, *button, *vbox, *sw;
+  gchar *str;
+  gint i;
+
+  if (!GTK_IS_CELL_LAYOUT (object))
+    return NULL;
+
+  cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (object));
+
+  table = gtk_table_new (g_list_length (cells), 2, FALSE);
+  gtk_table_set_col_spacing (GTK_TABLE (table), 0, 10);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 3);
+ 
+  for (c = cells, i = 0; c; c = c->next, i++)
+    {
+      object = c->data;
+
+      label = gtk_label_new ("Cell");
+      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+      gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, i, i + 1);
+
+      prop_edit = gtk_hbox_new (FALSE, 5);
+
+      str = object_label (object, NULL);
+      label = gtk_label_new (str);
+      g_free (str);
+      button = gtk_button_new_with_label ("Properties");
+      g_signal_connect (button, "clicked",
+                        G_CALLBACK (child_properties),
+                        object);
+
+      gtk_container_add (GTK_CONTAINER (prop_edit), label);
+      gtk_container_add (GTK_CONTAINER (prop_edit), button);
+
+      gtk_table_attach_defaults (GTK_TABLE (table), prop_edit, 1, 2, i, i + 1);
+    }
+
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+
+  sw = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), vbox);
+
+  g_list_free (cells);
+
+  return sw;
+}
 
 /* Pass zero for type if you want all properties */
 GtkWidget*
@@ -779,7 +1239,6 @@ create_prop_editor (GObject   *object,
 {
   GtkWidget *win;
   GtkWidget *notebook;
-  GtkTooltips *tips;
   GtkWidget *properties;
   GtkWidget *label;
   gchar *title;
@@ -796,10 +1255,6 @@ create_prop_editor (GObject   *object,
   if (GTK_IS_WIDGET (object))
     gtk_window_set_screen (GTK_WINDOW (win),
 			   gtk_widget_get_screen (GTK_WIDGET (object)));
-  
-  tips = gtk_tooltips_new ();
-  g_signal_connect_swapped (win, "destroy",
-			    G_CALLBACK (gtk_object_destroy), tips);
 
   /* hold a weak ref to the object we're editing */
   g_object_set_data_full (G_OBJECT (object), "prop-editor-win", win, model_destroy);
@@ -820,7 +1275,7 @@ create_prop_editor (GObject   *object,
       
       while (type)
 	{
-	  properties = properties_from_type (object, type, tips);
+	  properties = properties_from_type (object, type);
 	  if (properties)
 	    {
 	      label = gtk_label_new (g_type_name (type));
@@ -834,7 +1289,7 @@ create_prop_editor (GObject   *object,
       ifaces = g_type_interfaces (G_TYPE_FROM_INSTANCE (object), &n_ifaces);
       while (n_ifaces--)
 	{
-	  properties = properties_from_type (object, ifaces[n_ifaces], tips);
+	  properties = properties_from_type (object, ifaces[n_ifaces]);
 	  if (properties)
 	    {
 	      label = gtk_label_new (g_type_name (ifaces[n_ifaces]));
@@ -844,10 +1299,34 @@ create_prop_editor (GObject   *object,
 	}
 
       g_free (ifaces);
+
+      properties = child_properties_from_object (object);
+      if (properties)
+	{
+	  label = gtk_label_new ("Child properties");
+	  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+				    properties, label);
+	}
+
+      properties = children_from_object (object);
+      if (properties)
+	{
+	  label = gtk_label_new ("Children");
+	  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+				    properties, label);
+	}
+
+      properties = cells_from_object (object);
+      if (properties)
+	{
+	  label = gtk_label_new ("Cell renderers");
+	  gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
+				    properties, label);
+	}
     }
   else
     {
-      properties = properties_from_type (object, type, tips);
+      properties = properties_from_type (object, type);
       gtk_container_add (GTK_CONTAINER (win), properties);
       title = g_strdup_printf ("Properties of %s", g_type_name (type));
       gtk_window_set_title (GTK_WINDOW (win), title);
